import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
import os
import sys
import mrcfile
import scipy
import csv
from matplotlib.widgets import Slider
from scipy import ndimage
from scipy.signal import savgol_filter
from scipy.signal import find_peaks_cwt
from scipy.signal import find_peaks
from scipy.optimize import curve_fit
from numba import jit
from multiprocessing import Pool
from functools import partial

goldLattice = 2.347

def padNWPS(fft_width, pxGuess, latticeRes):
    #first find out how far to go
    radius_gold = fft_width*pxGuess/latticeRes
    #now I want to do the padding with zeros
    padWidth = (radius_gold*2) + (radius_gold*0.2)
    padXY = [0, 0]
    padXYVal = (padWidth - fft_width)
    if padXYVal % 2 == 0: #even
        padXY = [int(padXYVal/2), int(padXYVal/2)]
    else: #odd
        padXY = [int((padXYVal/2)-0.5), int((padXYVal/2)+0.5)]
    return padXY
    #NWPS_pad =  np.pad(NWPS, [padXY, padXY], 'constant')
    #return [NWPS_pad, padXY]


@jit(nopython=True)
def mirrorValues(NWPS_pad, fft_width, padXY):
    size = NWPS_pad.shape
    new_NWPS = NWPS_pad.copy()
    for i in range(size[0]):
        for j in range(size[0]):
            '''
            if i < padXY[0] or j < padXY[0]:
                new_NWPS[i, j] = new_NWPS[2*padXY[0]-i, 2*padXY[0]-j]
                new_NWPS[i+fft_width+padXY[0], j+fft_width+padXY[0]] = new_NWPS[padXY[0]+fft_width-i, padXY[0]+fft_width-j]
            '''
            #mirror the sides
            if i < padXY[0] and padXY[0] <= j < padXY[0] + fft_width: #left side
                new_NWPS[i, j] = new_NWPS[2*padXY[0]-i, j] #left side
                new_NWPS[padXY[0]+fft_width+i, j] = new_NWPS[padXY[0]+fft_width-i, j] #right side
            if j < padXY[0] and padXY[0] <= i < padXY[0] + fft_width:
                new_NWPS[i, j] = new_NWPS[i, 2*padXY[0]-j] #top side
                new_NWPS[i, padXY[0]+fft_width+j] = new_NWPS[i, padXY[0]+fft_width-j] #bottom side
            #mirror the corners
            if i < padXY[0] and j < padXY[0]:
               new_NWPS[i, j] = new_NWPS[2*padXY[0]-i, 2*padXY[0]-j]  
               new_NWPS[padXY[0]+fft_width+i, padXY[0]+fft_width+j] = new_NWPS[padXY[0]+fft_width-i, padXY[0]+fft_width-j]
               new_NWPS[padXY[0]+fft_width+i, j] = new_NWPS[padXY[0]+fft_width-i, 2*padXY[0]-j] 
               new_NWPS[i, padXY[0]+fft_width+j] = new_NWPS[2*padXY[0]-i, padXY[0]+fft_width-j] 
                
 
    return new_NWPS

def doAnImage(img, pxGuess, latticeRes):
    #need to get width to make power of 2 fft
    size = img.shape
    #now I want to mirror this outward
    padXY = padNWPS(size[0], pxGuess, latticeRes)
    NWPS_pad = np.pad(img, [padXY, padXY], 'constant')
    #Next set these to mirrored values
    new_NWPS = mirrorValues(NWPS_pad, size[0], padXY)
    return new_NWPS

def runScript(path, pxGuess, cores, latticeRes):
    #I want to check if the gold is beyond Nyquist 
    print("Dealing with aliasing")
    file_list = [f for f in os.listdir(path) if f.endswith(".mrc")]
    total = len(file_list)

    #load images
    NWPS = []
    for filename in file_list:
        mrc = mrcfile.open(path+filename, permissive=True)
        NWPS.append(mrc.data)
        mrc.close()

    #Process them
    p = Pool(processes=cores)
    doAnImage1 = partial(doAnImage, pxGuess=pxGuess, latticeRes=latticeRes)
    results = []
    for i, str_img in enumerate(p.map(doAnImage1, NWPS)):
        results.append(str_img)
        print(f"Image {str(i+1)}/{str(total)} processed")
    p.close()

    #write themuser_angles_temp.csv
    #make the new directory
    new_dir = "aliased"
    if not os.path.exists(f"{path}{new_dir}"):
        os.mkdir(f"{path}{new_dir}")
    print("Writing images")
    for i, img in enumerate(results):
        write_path = path + new_dir + '/' + file_list[i]
        with mrcfile.new(f"{write_path[:-4]}_aliased.mrc", overwrite=True) as mrc:
            mrc.set_data(np.float32(img))

    '''
    for i, filename in enumerate(file_list):
        mrc = mrcfile.open(path+filename, permissive=True)
        img = mrc.data
        mrc.close()
        #need to get width to make power of 2 fft
        size = img.shape
        #now I want to mirror this outward
        padXY = padNWPS(size[0], pxGuess)
        NWPS_pad = np.pad(img, [padXY, padXY], 'constant')
        #Next set these to mirrored values
        new_NWPS = mirrorValues(NWPS_pad, size[0], padXY)
            
        with mrcfile.new(path + filename[:-4] + "_aliased.mrc", overwrite=True) as mrc:
            mrc.set_data(np.float32(new_NWPS))
        print(f"Processed {i+1}/{total} spectra")

    '''









import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
import os
import sys
import mrcfile
import scipy
import csv
import pandas as pd
import math
from matplotlib.widgets import Slider
from matplotlib.widgets import TextBox
from matplotlib.widgets import Button
from scipy import ndimage
from scipy.signal import savgol_filter
from scipy.signal import find_peaks_cwt
from scipy.signal import find_peaks

#from matplotlib import rcParams
#rcParams['font.family'] = 'Helvetica'

px = 0
the_max = 0
savgol_window = 37

accept = False
reject = False
end = False

def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx

def radial_profile(data, center, angles):
    y,x = np.indices((data.shape)) # first determine radii of all pixels
    #yx = complex(y, x) #make complex
    all_angles = np.angle(x-center[0] + 1j*(y-center[1]), deg=True)+180 #get angles
    #excluded_indices = np.argwhere(all_angles < angles[0]) #find indices outside this angular range
    #to test

    r = np.sqrt((x-center[0])**2+(y-center[1])**2)
    ind = np.argsort(r.flat) # get sorted indices
    sr = r.flat[ind] # sorted radii

    sangles = all_angles.flat[ind] #sort angles in same way
    #included_indices = np.argwhere(sangles > angles[0])
    #print(angles[0])
    #print(angles[1])
    excluded_indices = np.argwhere(sangles < angles[0])
    excluded_indices2 = np.argwhere(sangles > angles[1])
     
    sim = data.flat[ind] # image values sorted by radii
    sim[excluded_indices] = 0 # image values sorted by radii
    sim[excluded_indices2] = 0 # image values sorted by radii
    ri = sr.astype(np.int32) # integer part of radii (bin size = 1)
    # determining distance between changes
    deltar = ri[1:] - ri[:-1] # assume all radii represented
    rind = np.where(deltar)[0] # location of changed radius
    nr = rind[1:] - rind[:-1] # number in radius bin
    csim = np.cumsum(sim, dtype=np.float64) # cumulative sum to figure out sums for each radii bin
    tbin = csim[rind[1:]] - csim[rind[:-1]] # sum for image values in radius bins
    radialprofile = tbin/nr # the answer
    #smoothed = savgol_filter(radialprofile, 99, 2)
    the_r = sr[rind]
    
    #return [abs(smoothed), the_r]
    return [abs(radialprofile), the_r]

def method2(img, pxGuess, latticeRes, angles, aliased):
    global accept, reject, end, the_max, px
    accept = False
    reject = False
    end = False
    size = img.shape
    leeway_plus = 0.05
    leeway_minus = 0.04
    size_alias =   2**(int(math.log(size[0], 2)))  
    #print(size[0])
    minradi = int((size[0]*(pxGuess-leeway_minus))/latticeRes)
    maxradi = int((size[0]*(pxGuess+leeway_plus))/latticeRes)
    if aliased == 'y':
        minradi = int((size_alias*(pxGuess-leeway_minus))/latticeRes)
        maxradi = int((size_alias*(pxGuess+leeway_plus))/latticeRes)   
    center, radi, radi2 = (size[0]/2, size[1]/2), minradi, maxradi

    result = radial_profile(img, center, angles)
    rad = result[0]
    smoothed = savgol_filter(rad, savgol_window, 2)
    ri = result[1]

    max_index = np.argmax(smoothed[radi:radi2])
    the_max = ri[max_index + radi]
    min_index = np.argmin(smoothed[radi:radi2])
    the_min = ri[min_index + radi]
    px = (the_max/(size[0]))*latticeRes
    if aliased == 'y':
        px = (the_max/(size_alias))*latticeRes

    mm = 1/25.4
    #fig = plt.figure(figsize=(45*mm, 30*mm))
    fig = plt.figure()
    ax = fig.subplots()
    ax.set_ylim(smoothed[min_index+radi], smoothed[max_index+radi] + 0.02*smoothed[max_index+radi])
    p, = ax.plot(ri[radi:radi2], smoothed[radi:radi2], 'k')
    p2, = ax.plot(the_max, smoothed[max_index+radi], "rx")
    plt.subplots_adjust(bottom=0.2, left=0.2, right=0.85, top=0.9)
    ax_slide = plt.axes([0.2, 0.08, 0.65, 0.03])
    win_size = Slider(ax_slide, 'Window size', valmin=3, valmax=99, valinit=savgol_window, valstep=2)
    right_side = ax.spines["right"]
    right_side.set_visible(False)
    top_side = ax.spines["top"]
    top_side.set_visible(False)
    for axis in ['left', 'bottom']:
        ax.spines[axis].set_linewidth(1)
        ax.spines[axis].set_color([0,0,0,0.6])
    ax.tick_params(axis='both', which = 'major', labelsize=7, color=[0,0,0,0.6], width=1)
    ax.tick_params(axis='both', which = 'minor', labelsize=7, color=[0,0,0,0.6], width=1)
    plt.xticks(alpha=0.6)
    plt.yticks(alpha=0.6)
    ax.set_xlabel("Radius (pixels)", fontsize=7.0, alpha=0.6)
    ax.set_ylabel("Intensity", fontsize=7.0, alpha=0.6)

    axbox_x = fig.add_axes([0.4, 0.02, 0.2, 0.04]) #left, bottom, width, height
    text_box_x = TextBox(axbox_x, "x_val", initial=str(round(the_max,1)))

    axAccept = plt.axes([0.1, 0.95, 0.2, 0.05])
    bAccept = Button(axAccept, 'Accept')
    axReject = plt.axes([0.4, 0.95, 0.2, 0.05])
    bReject = Button(axReject, 'Reject')
    axEnd = plt.axes([0.7, 0.95, 0.2, 0.05])
    bEnd = Button(axEnd, 'End')

    def submitx(expression):
        global px, the_max, savgol_window
        the_max = float(expression)
        px = (the_max/(size[0]))*latticeRes
        if aliased == 'y':
            px = (the_max/(size_alias))*latticeRes
        max_index = find_nearest(ri, the_max)
        the_max = ri[max_index]
        new_y = savgol_filter(rad, savgol_window, 2)
        p2.set_data(the_max, new_y[max_index])
        fig.canvas.draw()
    def update(val):
        global savgol_window, px
        current_v = int(win_size.val)
        savgol_window = current_v
        new_y = savgol_filter(rad, current_v, 2)
        max_index = np.argmax(new_y[radi:radi2])
        min_index = np.argmin(new_y[radi:radi2])
        the_max = ri[max_index + radi]
        the_min = ri[min_index + radi]
        px = (the_max/(size[0]))*latticeRes
        if aliased == 'y':
            px = (the_max/(size_alias))*latticeRes
        p.set_ydata(new_y[radi:radi2])
        p2.set_data(the_max, new_y[max_index+radi])
        ax.set_ylim(new_y[min_index+radi], new_y[max_index+radi] + 0.02*new_y[max_index+radi])
        fig.canvas.draw()
    def acceptClick(event):
        global accept, reject, end
        accept=True
        reject = False
        end=False
        plt.close()

    def rejectClick(event):
        global accept, reject, end
        accept=False
        reject = True
        end=False
        plt.close()

    def endClick(event):
        global accept, reject, end
        accept=False
        reject = False
        end=True
        plt.close()

    win_size.on_changed(update)
    text_box_x.on_submit(submitx)
    bAccept.on_clicked(acceptClick)
    bReject.on_clicked(rejectClick)
    bEnd.on_clicked(endClick)
    #plt.tight_layout()
    plt.show()

    
    #if (max_index <= 1):
    #    px = 0

    if reject == True:
        px = 0
    if end == True:
        px = 0

    return [px, str(end)]


if __name__ == "__main__":
    NWPS_dir = sys.argv[1]
    filename = sys.argv[2]
    write_dir = sys.argv[3]
    latticeRes = float(sys.argv[4])
    pxGuess = float(sys.argv[5])
    anglemin = int(sys.argv[6])
    anglemax = int(sys.argv[7])
    aliased = sys.argv[8]
    angles = (anglemin, anglemax)
    mrc = mrcfile.open(NWPS_dir+filename)
    img = mrc.data
    mrc.close()
    result = method2(img, pxGuess, latticeRes, angles, aliased)
    with open(f"{write_dir}user_px_temp.csv", 'w') as f:
        writer = csv.writer(f)
        writer.writerow(result)



import sys
import mrcfile
import numpy as np
import math
import csv
import os
from matplotlib.widgets import Slider
from matplotlib.widgets import Button
from scipy.signal import savgol_filter
import matplotlib.pyplot as plt
from numba import jit, njit
from multiprocessing import Pool
from functools import partial
import time
from PIL import Image, ImageEnhance
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure


savgol_window = 91
contrast_val = 2
coords = ''
anglemin=0
anglemax=180

cores = 8

accept = False
reject = False
end = False

def adjustContrast(data, c):
    thisQ = c*np.log10(1+data)
    return thisQ

def get_coords(x, y, angle, imwidth, imheight):
    endx1 = 0
    endx2 = imwidth
    endy1 = 0
    endy2 = imheight
    if angle == 0 or angle ==180:
        endy1 = imheight/2
        endy2 = imheight/2
    elif angle == 90 or angle ==270:
        endx1 = imwidth/2
        endx2 = imwidth/2
    else:
        x1_length = (x-imwidth) / math.cos(angle)
        y1_length = (y-imheight) / math.sin(angle)
        length = max(abs(x1_length), abs(y1_length))
        endx1 = x + length * math.cos(math.radians(angle))
        endy1 = y + length * math.sin(math.radians(angle))

        x2_length = (x-imwidth) / math.cos(angle+180)
        y2_length = (y-imheight) / math.sin(angle+180)
        length = max(abs(x2_length), abs(y2_length))
        endx2 = x + length * math.cos(math.radians(angle+180))
        endy2 = y + length * math.sin(math.radians(angle+180))

    return endx1, endy1, endx2, endy2

def getUserAngularRange(img, center, cores):
    global anglemin, anglemax, accept, reject, end
    accept = False
    reject = False
    end = False
    anglemin = 0
    anglemax = 180
    blank_range = [int(center[0]/20), int(center[1]/20)]
    mod_img = img.copy()
    mod_img[int(center[0]-blank_range[0]):int(center[0]+blank_range[0]), int(center[1]-blank_range[1]):int(center[1]+blank_range[1])] = 0
    size = img.shape
    max_val = np.amax(mod_img)
    c = 255 / (np.log10(1+max_val))

    #vecAdjust = np.vectorize(adjustContrast)
    #Q = vecAdjust(mod_img, c)

    #split up image and process bits on separate cores for speed-up
    split_images = np.array_split(mod_img, cores)
    p = Pool(processes=cores)
    adjustContrast1 = partial(adjustContrast, c=c)
    image_dict = p.map(adjustContrast1, split_images)
    p.close()
    Q = np.concatenate(image_dict)
    
    data = np.int8(Q)
    img_show = Image.fromarray(np.uint8(data), 'L')
    enhancer = ImageEnhance.Contrast(img_show)
    enhanced = enhancer.enhance(contrast_val)
    coords_angle = get_coords(center[0], center[1], 0, size[0], size[1])
    coords_angle2 = get_coords(center[0], center[1], 180, size[0], size[1])
    x = [coords_angle[0], coords_angle[2]]
    y = [coords_angle[1], coords_angle[3]]
    x2 = [coords_angle2[0], coords_angle2[2]]
    y2 = [coords_angle2[1], coords_angle2[3]]
    fig = plt.figure()
    ax = fig.subplots()
    p, = ax.plot(x, y, color="blue", linewidth=1, alpha=0.5)
    p2, = ax.plot(x2, y2, color="red", linewidth=1, alpha=0.5)
    #plt.imshow(data, cmap = 'gray')
    image_object = plt.imshow(enhanced, cmap = 'gray')
    plt.subplots_adjust(bottom=0.15, left=0.1, top=0.9)
    ax_slide = plt.axes([0.2, 0.05, 0.65, 0.03])
    win_size = Slider(ax_slide, 'Angle min', valmin=0, valmax=180, valinit=0, valstep=1, color='blue')
    ax_slide2 = plt.axes([0.2, 0.01, 0.65, 0.03])
    win_size2 = Slider(ax_slide2, 'Angle max', valmin=0, valmax=180, valinit=180, valstep=1, color='red')
    ax_slide_contrast = plt.axes([0.05, 0.25, 0.0225, 0.63])
    contrast_slider = Slider(ax_slide_contrast, 'Contrast', valmin=1, valmax=100, valinit=contrast_val, valstep=1, orientation="vertical")

    axAccept = plt.axes([0.1, 0.93, 0.2, 0.05])
    bAccept = Button(axAccept, 'Accept')
    axReject = plt.axes([0.4, 0.93, 0.2, 0.05])
    bReject = Button(axReject, 'Reject')
    axEnd = plt.axes([0.7, 0.93, 0.2, 0.05])
    bEnd = Button(axEnd, 'End')

    def on_press(event):
        sys.stdout.flush()
        global coords
        coords = event.key
        fig.canvas.mpl_disconnect(cid)
        plt.close(fig)
        return coords
    def update(val):
        global anglemin, anglemax
        anglemin = int(win_size.val)
        anglemax = int(win_size2.val)
        #print(anglemin)
        #print(anglemax)
        if anglemax < anglemin:
            win_size2.set_val(win_size.val+1)
            anglemax = anglemin+1
        coords_angle = get_coords(center[0], center[1], anglemin, size[0], size[1])
        coords_angle2 = get_coords(center[0], center[1], anglemax, size[0], size[1])
        x = [coords_angle[0], coords_angle[2]]
        y = [coords_angle[1], coords_angle[3]]
        x2 = [coords_angle2[0], coords_angle2[2]]
        y2 = [coords_angle2[1], coords_angle2[3]]
        #plt.clf()
        p.set_data(x,y)
        p2.set_data(x2,y2)
        fig.canvas.draw()
    def update_contrast(val):
        global contrast_val
        contrast_val = int(contrast_slider.val)
        enhanced = enhancer.enhance(contrast_val)
        image_object.set_data(enhanced)
        fig.canvas.draw()

    def acceptClick(event):
        global accept, reject, end
        accept=True
        reject = False
        end=False
        plt.close()

    def rejectClick(event):
        global accept, reject, end
        accept=False
        reject = True
        end=False
        plt.close()

    def endClick(event):
        global accept, reject, end
        accept=False
        reject = False
        end=True
        plt.close()
    win_size.on_changed(update)
    win_size2.on_changed(update)
    contrast_slider.on_changed(update_contrast)
    cid = fig.canvas.mpl_connect('key_press_event', on_press)
    bAccept.on_clicked(acceptClick)
    bReject.on_clicked(rejectClick)
    bEnd.on_clicked(endClick)
    plt.show()
    
    angles = [anglemin, anglemax]
    #print(anglemin)
    #print(anglemax)
    return [anglemin, anglemax, str(accept), str(reject), str(end)]

if __name__ == "__main__":
    NWPS_dir = sys.argv[1]
    filename = sys.argv[2]
    cores = int(sys.argv[3])
    write_dir = sys.argv[4]
    mrc = mrcfile.open(NWPS_dir+filename)
    img = mrc.data
    mrc.close()
    size = img.shape  
    center = (size[0]/2, size[1]/2)
    result = getUserAngularRange(img, center, cores)
    
    with open(f"{write_dir}user_angles_temp.csv", 'w') as f:
        writer = csv.writer(f)
        writer.writerow(result)




import numpy as np
from scipy import ndimage
import mrcfile
import os
import sys
#import cv2
from skimage.transform import resize
from multiprocessing import Pool
from functools import partial

def stretchImage(img, NWPS_dir, stretch_dir, aniso_axis, a, angle_a, angle_b):
    '''
    #switch this to be in single thread I think
    mrc = mrcfile.open(NWPS_dir+filename)
    img = mrc.data
    mrc.close()
    '''


    #Then pad it to allow a rotation
    size = img.shape
    new_width = (size[0]**2 + size[1]**2)**0.5
    padVal = int((new_width - size[0])/2)
    padded_img = np.pad(img, [[padVal, padVal], [padVal, padVal]], 'constant')

    #Rotate it to put axis to be stretched or squeezed into x

    img_rot = ndimage.rotate(padded_img, aniso_axis, reshape=False)

    #Apply the squeeze or stretch - this goes to a new array that starts with zeros same shape

    '''
    stretch_matrix = [[a, 0],[0, 1]]
    shrink_matrix = [[1/a, 0],[0, 1]]
    blank_img = np.zeros_like(img_rot)
    shrunk_img = np.dot(shrink_matrix, img_rot)
    stretch_img = np.dot(stretch_matrix, img_rot)
    '''

    #vis2 = cv2.cvtColor(img_rot, cv2.COLOR_GRAY2BGR)
    #vis2 = img_rot.copy()
    #height = int(img_rot.shape[0])
    #width = int(img_rot.shape[1] * a)
    height = int(img_rot.shape[0] * a)
    width = int(img_rot.shape[1])
    dim = (width, height)
    #resized_img = cv2.resize(img_rot, dim, interpolation = cv2.INTER_LINEAR)
    resized_img = resize(img_rot, dim)
    resized_img_np = np.asarray(resized_img)#convert back to numpy

    #rotate the images back
    #shrunk_img = ndimage.rotate(shrunk_img, aniso_axis*-1.0, reshape=False)
    stretch_img = ndimage.rotate(resized_img_np, aniso_axis*-1.0, reshape=False)

    #Now slice back to a normal size
    size2 = stretch_img.shape
    centre_width = size2[1]/2
    centre_height = size2[0]/2
    start_width = int(centre_width-(size[1]/2))
    end_width = int(centre_width+(size[1]/2))
    start_height = int(centre_height-(size[0]/2))
    end_height = int(centre_height+(size[0]/2))

    #need to make sure still a power of 2 after this slicing
    stretch_img2 = stretch_img[start_height:end_height, start_width:end_width]

    return stretch_img2

    #save fft
    '''
    #might just return this and do later one at a time, no way to speed io
    with mrcfile.new(f"{stretch_dir}/{filename[:-4]}_stretch.mrc, overwrite=True) as mrc:
        mrc.set_data(np.float32(stretch_img2))
    '''


def runScript(path, cores, spectra_dir, aliased, ps_dir, write_dir, aniso_file, avgImage):
    print_text = []
    #get directory
    #NWPS_dir = f"{path}{spectra_dir}/avg/10/"
    NWPS_dir = ps_dir
    #make new dirs
    stretch_dir = f"{write_dir}stretch"
    if not os.path.exists(stretch_dir):
        os.mkdir(stretch_dir)
    if avgImage:
        stretch_dir += "/avg"
        if not os.path.exists(stretch_dir):
            os.mkdir(stretch_dir)
    file_list = []

    '''
    if aliased == False:
        file_list = [f for f in os.listdir(NWPS_dir) if (f.endswith(".mrc") and f.endswith("aliased.mrc") == False)]
    else:
        file_list = [f for f in os.listdir(NWPS_dir) if f.endswith("aliased.mrc")]
    num_files = len(file_list)
    if num_files < 1:
        NWPS_dir = f"{path}{spectra_dir}/avg/"
        if aliased == False:
            file_list = [f for f in os.listdir(NWPS_dir) if (f.endswith(".mrc") and f.endswith("aliased.mrc") == False)]
        else:
            file_list = [f for f in os.listdir(NWPS_dir) if f.endswith("aliased.mrc")]
        num_files = len(file_list)

    #make new dirs
    stretch_dir = f"{NWPS_dir}stretch"
    if not os.path.exists(stretch_dir):
            os.mkdir(stretch_dir)
    '''
    NWPS_dir = ps_dir
    file_list = [f for f in os.listdir(NWPS_dir) if f.endswith(".mrc")]
    num_files = len(file_list)
    #get aniso file
    print("Loading anisotropy parameters")
    print_text.append("Loading anisotropy parameters")
    line = ""
    #with open(f"{NWPS_dir}aniso_params.csv") as f:
    with open(aniso_file) as f:
        line = f.readline()
    l = line.split(',')
    a = float(l[0])/float(l[1])
    angle_a = float(l[2])
    angle_b = angle_a - np.pi/2
    if angle_b < 0:
        angle_b = np.pi + angle_b
    aniso_axis = np.degrees(angle_b)

    print("Loading power spectra")
    print_text.append("Loading power spectra")
    #load images
    NWPS = []
    for filename in file_list:
        mrc = mrcfile.open(NWPS_dir+filename)
        NWPS.append(mrc.data)
        mrc.close()

    print("Beginning stretching images")
    print_text.append("Beginning stretching images")
    #Process them
    p = Pool(processes=cores)
    stretchImage1 = partial(stretchImage, NWPS_dir=NWPS_dir, stretch_dir=stretch_dir, aniso_axis=aniso_axis, a=a, angle_a=angle_a, angle_b=angle_b)
    results = []
    for i, str_img in enumerate(p.imap_unordered(stretchImage1, NWPS)):
        results.append(str_img)
        print(f"Image {str(i+1)}/{str(num_files)} complete")
        print_text.append(f"Image {str(i+1)}/{str(num_files)} complete")
    p.close()

    #write them
    print("Writing images")
    print_text.append("Writing images")
    for i, img in enumerate(results):
        with mrcfile.new(f"{stretch_dir}/stretchNWPS_{str(i+1)}.mrc", overwrite=True) as mrc:
                mrc.set_data(np.float32(img))

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")
        






import os
import sys

def removeWhitespace(path, filenames):
    for filename in filenames:
        os.rename(os.path.join(path, filename), os.path.join(path, filename.replace(" ", "-")))

def convertDM2MRC(path, extension, filenames):
    removeWhitespace(path, filenames)
    for filename in os.listdir(path):
        if filename.endswith(f".{extension}"):
            os.system(f"dm2mrc {path}{filename} {path}{filename[:-4]}.mrc")
    

def convertTIFF2MRC(path, extension, filenames):
    removeWhitespace(path, filenames)
    for filename in os.listdir(path):
        if filename.endswith(".tif"):
            os.system(f"tif2mrc {path}{filename} {path}{filename[:-4]}.mrc")
        if filename.endswith(".tiff"):
            os.system(f"tif2mrc {path}{filename} {path}{filename[:-5]}.mrc")

def runConvert(path, extension):
    filenames = os.listdir(path)
    if extension == "dm3" or extension == "dm4":
        convertDM2MRC(path, extension, filenames)
    if extension == "tiff":
        convertTIFF2MRC(path, extension, filenames)




import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
import os
import sys
import mrcfile
import scipy
import csv
from matplotlib.widgets import Slider
from scipy import ndimage
from scipy.signal import savgol_filter
from scipy.signal import find_peaks_cwt
from scipy.signal import find_peaks
from scipy.optimize import curve_fit
from numba import jit
from multiprocessing import Pool
from functools import partial

coords = ''   

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def getMTFAll(MTF_file, pxGuess):
    #MTF_file = "data/mtf_falcon4EC_300kV.star"
    freq = []
    MTF = []
    f = open(MTF_file, 'r')
    for line in f.readlines():
        l = line.replace(' ',"\t").split("\t")
        if len(l) > 1:
            if is_number(l[0]):
                this_freq = (float(l[0])/pxGuess)
                freq.append(this_freq)
                MTF.append(float(l[1])**2)
    f.close()
    return [freq, MTF]

@jit(nopython=True)
def hyperbolicFunction(x, alpha, delta, theta):
    return (-1/(1+alpha*np.exp(-delta*x-theta*np.arcsinh(x))))+1

def gaussianFunction(x, a, c):
    return a*np.exp((-((x-0)**2)/(2*c**2)))

def fitHyperbolic(detectorMTF):
    MTF_freqs = np.array(detectorMTF[0])
    MTF_vals = np.array(detectorMTF[1])
    #popt, pcov = curve_fit(gaussianFunction, MTF_freqs, MTF_vals, p0=[1.0, 0.16])
    popt, pcov = curve_fit(hyperbolicFunction, MTF_freqs, MTF_vals, p0=[22.685, -77.966, 95.384])
    return popt

def extrapolate(detectorMTF):
    MTF_freqs = np.array(detectorMTF[0])
    MTF_vals = np.array(detectorMTF[1])
    grad = (MTF_vals[-1] - MTF_vals[-2])/(MTF_freqs[-1] - MTF_freqs[-2])
    c = MTF_vals[-1] - grad*MTF_freqs[-1]
    return [grad, c]

@jit(nopython=True)
def loopFreqs(power, FreqCompx, FreqCompy, MTF_freqs, max_freq, MTF_vals):
    NWPS = np.zeros_like(power)
    for i in range(len(FreqCompx)):
        for j in range(len(FreqCompy)):
            freq = (FreqCompx[i]**2+FreqCompy[j]**2)**0.5
            #now interpolate
            if freq < max_freq:
                this_val = np.interp(freq, MTF_freqs, MTF_vals)
                NWPS[i, j] = power[i, j]/this_val
            else:
                #this_val = gaussianFunction(freq, popt[0], popt[1])
                #this_val = hyperbolicFunction(freq, popt[0], popt[1], popt[2])
                #this_val = freq*popt[0] + popt[1]
                this_val = np.interp(max_freq, MTF_freqs, MTF_vals)
                NWPS[i, j] = power[i, j]/this_val
    return NWPS
    


def getNWPS(power, detectorMTF, pxGuess):

    FreqCompx = np.fft.fftfreq(power.shape[0], pxGuess)
    FreqCompy = np.fft.fftfreq(power.shape[1], pxGuess)
    MTF_freqs = np.array(detectorMTF[0])
    max_freq = np.max(MTF_freqs)
    MTF_vals = np.array(detectorMTF[1])
    '''
    NWPS = np.zeros_like(power)

    for i in range(len(FreqCompx)):
        for j in range(len(FreqCompy)):
            freq = (FreqCompx[i]**2+FreqCompy[j]**2)**0.5
            #now interpolate
            if freq < max_freq:
                this_val = np.interp(freq, MTF_freqs, MTF_vals)
                NWPS[i, j] = power[i, j]/this_val
            else:
                #this_val = gaussianFunction(freq, popt[0], popt[1])
                this_val = hyperbolicFunction(freq, popt[0], popt[1], popt[2])
                NWPS[i, j] = power[i, j]/this_val
    '''
    NWPS = loopFreqs(power, FreqCompx, FreqCompy, MTF_freqs, max_freq, MTF_vals)
    NWPS_2= np.fft.fftshift(NWPS)
    return NWPS_2


#if __name__ == "__main__":
def runScript(path, MTF_file, pxGuess, cores, spectra_dir, write_dir, num_images):
    print_text = []
    print("Noise whitening using the MTF")
    print_text.append("Noise whitening using the MTF")
    detectorMTF = getMTFAll(MTF_file, pxGuess)
    #MTF_params = fitHyperbolic(detectorMTF)
    #MTF_params = np.asarray(extrapolate(detectorMTF))
    count = 0
    NWPS_sum = []
    new_dir = f"{write_dir}{spectra_dir}"
    if not os.path.exists(new_dir):
            os.mkdir(new_dir)
            os.mkdir(f"{new_dir}/avg")
            os.mkdir(f"{new_dir}/avg/10")

    file_list = [f for f in os.listdir(path) if (f.endswith(".mrc") and f.endswith("_PS.mrc") == False)]
    cryosparc = False
    for files in file_list:
        if "patch_aligned" in files or "rigid_aligned" in files:
            cryosparc = True
            break
    file_list2 = []
    if cryosparc == True:
        file_list2 = [f for f in file_list if ("patch_aligned" in f or "rigid_aligned" in f) and (f.endswith("aligned_doseweighted.mrc")==False)] 
        file_list = file_list2
    num_files = len(file_list)
    images = 0
    if num_images <= 0 or num_images > num_files:
        images = num_files
    else:
        images = num_images
    
    files_per_sum = int(images/10)
    NWPS_sum10 = []
    count_10 = 0
    count_10_2 = 0
    for filename in file_list:
        if filename.endswith(".mrc"):
            mrc = mrcfile.open(path+filename, permissive=True)
            img = mrc.data
            mrc.close()
            #need to get width to make power of 2 fft
            size = img.shape
            powerOfTwo_y = np.ceil(np.log2(size[0]))
            powerOfTwo_x = np.ceil(np.log2(size[1])) 
            powerOfTwo = powerOfTwo_y
            if powerOfTwo_x > powerOfTwo:
                powerOfTwo = powerOfTwo_x
            fft_width = int(2**powerOfTwo)
            power = np.absolute(np.fft.fft2(img, s=(fft_width, fft_width)))**2
            #power = np.absolute(np.fft.fft2(img))**2
            NWPS = getNWPS(power, detectorMTF, pxGuess)
            if count == 0:
                NWPS_sum = NWPS.copy()
            else:
                NWPS_sum = np.add(NWPS_sum, NWPS)
            count += 1
            if count_10 == 0:
                NWPS_sum10 = NWPS.copy()
            else:
                NWPS_sum10 = np.add(NWPS_sum10, NWPS)
            count_10 += 1
            if count_10 >= 10:
                count_10 = 0
                count_10_2 += 1
                #write file
                with mrcfile.new(f"{write_dir}NWPS/avg/10/avgNWPS10_{str(count_10_2)}.mrc", overwrite=True) as mrc:
                    mrc.set_data(np.float32(NWPS_sum10)) 
                #reset NWPS_10
                NWPS_sum10 = []
            
            with mrcfile.new(f"{write_dir}NWPS/{filename[:-4]}_NWPS.mrc", overwrite=True) as mrc:
                mrc.set_data(np.float32(NWPS))
            print(f"Completed file {str(count)}/{str(images)}")
            print_text.append(f"Completed file {str(count)}/{str(images)}")
            if (count+1)>= images:
                break

    print("Writing sum spectrum")
    print_text.append("Writing sum spectrum")
    with mrcfile.new(f"{write_dir}NWPS/avg/avgNWPS_MTF.mrc", overwrite=True) as mrc:
        mrc.set_data(np.float32(NWPS_sum))

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")
    










import sys
import mrcfile
import numpy as np
import os
#import aliasPS

def runScript(path, filename, new_dir):
    #load mrc
    mrc = mrcfile.open(path + filename, permissive=True)
    data=mrc.data
    mrc.close()
    #work out final FFT size
    size = data.shape
    powerOfTwo_y = np.ceil(np.log2(size[0]))
    powerOfTwo_x = np.ceil(np.log2(size[1])) 
    powerOfTwo = powerOfTwo_y
    if powerOfTwo_x > powerOfTwo:
        powerOfTwo = powerOfTwo_x
    fft_width = int(2**powerOfTwo)
    #make PS
    power = np.absolute(np.fft.fftshift(np.fft.fft2(data, s=(fft_width, fft_width))))**2
    #write file
    with mrcfile.new(f"{new_dir}/{filename[:-4]}_PS.mrc", overwrite=True) as mrc:
        mrc.set_data(np.float32(power))
    return power
    
        




        



import os
import sys
import csv
import numpy as np
import scipy.stats as st

def getDirectory(path, spectra_dir):
    NWPS_dir = f"{path}{spectra_dir}/avg/10/"
    file_list = [f for f in os.listdir(NWPS_dir) if f.endswith(".mrc")]
    num_files = len(file_list)
    if num_files < 1:
        NWPS_dir = f"{path}{spectra_dir}/avg/"
    return NWPS_dir

#def runScript(path, aniso_corrected, spectra_dir, write_dir, aniso_file):
def runScript(path, spectra_dir, write_dir, aniso_file):
    print_text = []
    #get directory
    #directory = getDirectory(path, spectra_dir)
    aniso_corrected = True
    if aniso_file == "":
        aniso_corrected = False
    
    ratio = 1.0
    if aniso_corrected:
        with open(aniso_file, 'r') as f:
            line = f.readline()
            l = line.split(',')
            ratio = float(l[1])/float(l[0])

    #and now read the pixel sizes
    '''
    directory2 = directory
    if aniso_corrected:
        directory2 = f"{directory}stretch/"
    all_files = os.listdir(directory2)
    the_file = ''
    for filename in all_files:
        if filename.endswith(".csv") and "px" in filename and "smooth" in filename:
            the_file = filename
            break
    '''
    overall = []
    px_vals = []
    #with open(directory2+the_file, 'r') as f:
    with open(f"{write_dir}pxSizes.csv", 'r') as f:
        for line in f.readlines():
            l = line.split(',')
            if len(l) > 1:
                if float(l[1]) > 0:
                    #val = (2*float(l[1]) + ratio*float(l[1]))/3
                    #val = (float(l[1]) * (ratio*float(l[1])))**0.5
                    val = ((float(l[1])**2 + (ratio*float(l[1]))**2)/2)**0.5
                    overall.append([l[0], val])
                    if aniso_corrected:
                        px_vals.append(val)
                    else:
                        px_vals.append(float(l[1]))
         
    if aniso_corrected:
        #now write
        with open(write_dir+"pxSizes_adjust.csv", 'w') as f:
            writer = csv.writer(f)
            writer.writerows(overall)

    #Now it time for the stats
    #now remove any outliers if more than a certain amoumy
    px_sizes = []
    if len(px_vals) >= 3:
        px_vals = np.array(px_vals)
        mean_px = np.mean(px_vals)
        stdev = np.std(px_vals)
        d_from_mean = abs(px_vals-mean_px)
        max_deviations = 2
        not_outlier = d_from_mean < max_deviations * stdev
        px_sizes = px_vals[not_outlier]
    else:
        px_sizes = np.array(px_vals)


    mean_px = np.mean(px_sizes)
    print("Mean px: " + str(round(mean_px, 4)))
    print_text.append("Mean px: " + str(round(mean_px, 4)))

    if len(px_vals) >= 3:
        #interval_px = st.t.interval(alpha=0.95, df=len(px_sizes)-1, loc=mean_px, scale=st.sem(px_sizes))
        interval_px = st.t.interval(0.95, len(px_sizes)-1, loc=mean_px, scale=st.sem(px_sizes))

        print("Interval px size: " + str(interval_px))
        print_text.append("Interval px size: " + str(interval_px))
        conf = mean_px - interval_px[0]
        print("Conf px size: " + str(conf))
        print_text.append("Conf px size: " + str(conf))


        with open(write_dir+"stats.csv", 'w') as f:
            f.write(f"{mean_px},{interval_px[0]},{interval_px[1]}")
    else:
        with open(write_dir+"stats.csv", 'w') as f:
            f.write(f"{mean_px}")

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")







import sys
import mrcfile
import numpy as np
import math
import csv
import os
from matplotlib.widgets import Slider
from scipy.signal import savgol_filter
import matplotlib.pyplot as plt
from numba import jit, njit
from multiprocessing import Pool
from functools import partial
import time
from PIL import Image, ImageEnhance

#path = sys.argv[1]
savgol_window = 91
contrast_val = 2
coords = ''

def adjustContrast(data, c):
    thisQ = c*np.log10(1+data)
    return thisQ

def get_coords(x, y, angle, imwidth, imheight):
    endx1 = 0
    endx2 = imwidth
    endy1 = 0
    endy2 = imheight
    if angle == 0 or angle ==180:
        endy1 = imheight/2
        endy2 = imheight/2
    elif angle == 90 or angle ==270:
        endx1 = imwidth/2
        endx2 = imwidth/2
    else:
        x1_length = (x-imwidth) / math.cos(angle)
        y1_length = (y-imheight) / math.sin(angle)
        length = max(abs(x1_length), abs(y1_length))
        endx1 = x + length * math.cos(math.radians(angle))
        endy1 = y + length * math.sin(math.radians(angle))

        x2_length = (x-imwidth) / math.cos(angle+180)
        y2_length = (y-imheight) / math.sin(angle+180)
        length = max(abs(x2_length), abs(y2_length))
        endx2 = x + length * math.cos(math.radians(angle+180))
        endy2 = y + length * math.sin(math.radians(angle+180))

    return endx1, endy1, endx2, endy2

def getUserAngularRange(img, center, cores):
    blank_range = [int(center[0]/20), int(center[1]/20)]
    mod_img = img.copy()
    mod_img[int(center[0]-blank_range[0]):int(center[0]+blank_range[0]), int(center[1]-blank_range[1]):int(center[1]+blank_range[1])] = 0
    size = img.shape
    max_val = np.amax(mod_img)
    c = 255 / (np.log10(1+max_val))

    #vecAdjust = np.vectorize(adjustContrast)
    #Q = vecAdjust(mod_img, c)

    #split up image and process bits on separate cores for speed-up
    split_images = np.array_split(mod_img, cores)
    p = Pool(processes=cores)
    adjustContrast1 = partial(adjustContrast, c=c)
    image_dict = p.map(adjustContrast1, split_images)
    p.close()
    Q = np.concatenate(image_dict)
    
    data = np.int8(Q)
    img_show = Image.fromarray(np.uint8(data), 'L')
    enhancer = ImageEnhance.Contrast(img_show)
    enhanced = enhancer.enhance(contrast_val)
    coords_angle = get_coords(center[0], center[1], 0, size[0], size[1])
    coords_angle2 = get_coords(center[0], center[1], 180, size[0], size[1])
    x = [coords_angle[0], coords_angle[2]]
    y = [coords_angle[1], coords_angle[3]]
    x2 = [coords_angle2[0], coords_angle2[2]]
    y2 = [coords_angle2[1], coords_angle2[3]]
    fig = plt.figure()
    ax = fig.subplots()
    p, = ax.plot(x, y, color="blue", linewidth=1, alpha=0.5)
    p2, = ax.plot(x2, y2, color="red", linewidth=1, alpha=0.5)
    #plt.imshow(data, cmap = 'gray')
    image_object = plt.imshow(enhanced, cmap = 'gray')
    plt.subplots_adjust(bottom=0.15, left=0.1)
    ax_slide = plt.axes([0.2, 0.05, 0.65, 0.03])
    win_size = Slider(ax_slide, 'Angle min', valmin=0, valmax=180, valinit=0, valstep=1, color='blue')
    ax_slide2 = plt.axes([0.2, 0.01, 0.65, 0.03])
    win_size2 = Slider(ax_slide2, 'Angle max', valmin=0, valmax=180, valinit=180, valstep=1, color='red')
    ax_slide_contrast = plt.axes([0.05, 0.25, 0.0225, 0.63])
    contrast_slider = Slider(ax_slide_contrast, 'Contrast', valmin=1, valmax=100, valinit=contrast_val, valstep=1, orientation="vertical")

    def on_press(event):
        sys.stdout.flush()
        global coords
        coords = event.key
        fig.canvas.mpl_disconnect(cid)
        plt.close(fig)
        return coords
    def update(val):
        global anglemin, anglemax
        anglemin = int(win_size.val)
        anglemax = int(win_size2.val)
        #print(anglemin)
        #print(anglemax)
        if anglemax < anglemin:
            win_size2.set_val(win_size.val+1)
            anglemax = anglemin+1
        coords_angle = get_coords(center[0], center[1], anglemin, size[0], size[1])
        coords_angle2 = get_coords(center[0], center[1], anglemax, size[0], size[1])
        x = [coords_angle[0], coords_angle[2]]
        y = [coords_angle[1], coords_angle[3]]
        x2 = [coords_angle2[0], coords_angle2[2]]
        y2 = [coords_angle2[1], coords_angle2[3]]
        #plt.clf()
        p.set_data(x,y)
        p2.set_data(x2,y2)
        fig.canvas.draw()
    def update_contrast(val):
        global contrast_val
        contrast_val = int(contrast_slider.val)
        enhanced = enhancer.enhance(contrast_val)
        image_object.set_data(enhanced)
        fig.canvas.draw()
    win_size.on_changed(update)
    win_size2.on_changed(update)
    contrast_slider.on_changed(update_contrast)
    cid = fig.canvas.mpl_connect('key_press_event', on_press)
    plt.show()
    
    angles = (anglemin, anglemax)
    #print(anglemin)
    #print(anglemax)
    return angles

@njit
def loopImage(size, x, y):
    
    #make an array of ones of same size as image
    write_mrc = np.ones(size)
    #print(size)
    centre = (int(size[0]/2), int(size[1]/2))
    max_distance = np.max(x)
    #go through each pixel and get the distance to centre
    for h in range(size[0]):
        for w in range(size[1]):
            #get distance to centre
            distance = ((centre[0]-h)**2 + (centre[1]-w)**2)**0.5
            #interpolate this value in profile if it can be done
            if distance <= max_distance:
                write_mrc[h][w] = np.interp(distance, x, y)
            else:
                write_mrc[h][w] = np.interp(max_distance, x, y)
    
    return write_mrc
                

def radial_profile(data, center, angles):
    y,x = np.indices((data.shape)) # first determine radii of all pixels
    #yx = complex(y, x) #make complex
    all_angles = np.angle(x-center[0] + 1j*(y-center[1]), deg=True)+180 #get angles
    #excluded_indices = np.argwhere(all_angles < angles[0]) #find indices outside this angular range
    #to test

    r = np.sqrt((x-center[0])**2+(y-center[1])**2)
    ind = np.argsort(r.flat) # get sorted indices
    sr = r.flat[ind] # sorted radii

    sangles = all_angles.flat[ind] #sort angles in same way
    #included_indices = np.argwhere(sangles > angles[0])
    #print(angles[0])
    #print(angles[1])
    excluded_indices = np.argwhere(sangles < angles[0])
    excluded_indices2 = np.argwhere(sangles > angles[1])
     
    sim = data.flat[ind] # image values sorted by radii
    sim[excluded_indices] = 0 # image values sorted by radii
    sim[excluded_indices2] = 0 # image values sorted by radii
    ri = sr.astype(np.int32) # integer part of radii (bin size = 1)
    # determining distance between changes
    deltar = ri[1:] - ri[:-1] # assume all radii represented
    rind = np.where(deltar)[0] # location of changed radius
    nr = rind[1:] - rind[:-1] # number in radius bin
    csim = np.cumsum(sim, dtype=np.float64) # cumulative sum to figure out sums for each radii bin
    tbin = csim[rind[1:]] - csim[rind[:-1]] # sum for image values in radius bins
    radialprofile = tbin/nr # the answer
    #smoothed = savgol_filter(radialprofile, 99, 2)
    the_r = sr[rind]
    
    #return [abs(smoothed), the_r]
    return [abs(radialprofile), the_r]

def makeRaPowerSpectrum(blank_img, fft_width):
    noise_power = np.absolute(np.fft.fftshift(np.fft.fft2(blank_img, s=(int(fft_width), int(fft_width)))))**2
    #now I want to do the radial profile
    size = noise_power.shape
    center = (size[0]/2, size[1]/2)
    angles = (0, 180)
    result = radial_profile(noise_power, center, angles)
    rad = result[0]
    ri = result[1][:-1]
    return [rad, ri]

def runScript(path, cores, choose_angles, write_dir, px_guess, latticeType, latticeRes, temperature, num_images):
    new_filename = f"{write_dir}NPS_ra_smooth{str(savgol_window)}.mrc"
    #if not os.path.exists(new_filename):
    theRealProcessing(path, cores, new_filename, choose_angles, write_dir, px_guess, latticeType, latticeRes, temperature, num_images)


def theRealProcessing(path, cores, new_filename, choose_angles, write_dir, px_guess, latticeType, latticeRes, temperature, num_images):
    print_text = []
    print("Making power spectrum from images, ignoring the gold peak")
    print_text.append("Making power spectrum from images, ignoring the gold peak")
    noise_data = []
    print("Loading images")
    print_text.append("Loading images")
    file_list = [f for f in os.listdir(path) if (f.endswith(".mrc")==True and f.endswith("_PS.mrc") == False)]
    cryosparc = False
    for files in file_list:
        if "patch_aligned" in files or "rigid_aligned" in files:
            cryosparc = True
            break
    file_list2 = []
    if cryosparc == True:
        file_list2 = [f for f in file_list if ("patch_aligned" in f or "rigid_aligned" in f) and (f.endswith("aligned_doseweighted.mrc")==False)] 
        file_list = file_list2
    num_files = len(file_list)
    images = 0
    if num_images <= 0 or num_images > num_files:
        images = num_files
    else:
        images = num_images
    #update file_list
    file_list = file_list[:images]
    for filename in file_list:
        mrc_blank = mrcfile.open(path + filename, permissive=True)
        noise_data.append(mrc_blank.data)
        mrc_blank.close()
    print("Images loaded")
    print_text.append("Images loaded")
    size = noise_data[0].shape
    powerOfTwo_y = np.ceil(np.log2(size[0]))
    powerOfTwo_x = np.ceil(np.log2(size[1])) 
    powerOfTwo = powerOfTwo_y
    if powerOfTwo_x > powerOfTwo:
        powerOfTwo = powerOfTwo_x
    fft_width = int(2**powerOfTwo)
    new_size = (fft_width, fft_width)
    noise_power = []
    count = len(noise_data)
    x = []
    y = []
    
    angles = (0, 180)

        
        
    
    
    #I should pass the angles in to make the raw power spectrum
    #Then I can have an option is user choose angles or not 
    #If the user chooses angles, then just add in the choice of angles bit
    #The crop option is a similar sort of thing
    #Crop and angle just combineLEARN
    #Therefore all the options, except MTF, should be in this script
    #MTF i'll need to change the label name from whitening directory to MTF file
    #Name should just switch from MTF file to blank image dir and back instead of whitening dir

    start_time = time.time()
    
    p = Pool(processes=cores)
    makeRaPowerSpectrum1 = partial(makeRaPowerSpectrum, fft_width=fft_width)
    #results = p.map(makeRaPowerSpectrum1, noise_data)
    results = []
    print("Calculating radial profile of FFTs...")
    print_text.append("Calculating radial profile of FFTs...")
    for i, return_result in enumerate(p.imap_unordered(makeRaPowerSpectrum1, noise_data)):
        results.append(return_result)
        fraction_done = round(((i+1)/count),2)*100
        print(f"Finished {fraction_done}% of files")
        print_text.append(f"Finished {fraction_done}% of files")
        if (i+1)>= images:
            break
    p.close()
    x = results[0][1]
    sum_all = np.mean(results, axis=0)
    y = sum_all[0]
    
    '''
    for i in range(count):
        noise_power = np.absolute(np.fft.fftshift(np.fft.fft2(noise_data[i], s=(int(fft_width), int(fft_width)))))**2
        #now I want to do the radial profile
        size = noise_power.shape
        center = (size[0]/2, size[1]/2)
        if choose_angles:
            angles = getUserAngularRange(noise_power, center, cores)
        result = radial_profile(noise_power, center, angles)
        rad = result[0]
        #smoothed = savgol_filter(rad, savgol_window, 2)
        ri = result[1][:-1]
        if i == 0:
            x = ri
            y = rad.copy()
        else:
            y = np.add(y, rad)
        
    y /= count
    '''

    print("Power spectra radial profiles calculated --- %s seconds ---" % round(time.time() - start_time,2))
    print_text.append("Power spectra radial profiles calculated --- %s seconds ---" % round(time.time() - start_time,2))

    #now I want to block out the stuff around px_guess before smoothing!
    gold_lattice_param = temperature*5.67075E-5 + 4.06111 #A
    #adjust for very low temperature
    if temperature < 43.0:
        gold_lattice_param = 43.0*5.67075E-5 + 4.06111 #A       
    goldLattice_111 = gold_lattice_param / (1**2+1**2+1**2)**0.5
    goldLattice_200 = gold_lattice_param / (2**2)**0.5
    #gc_res = 3.3378
    gc_res = 3.4187
    print(f"Lattice type is {latticeType}")
    print_text.append(f"Lattice type is {latticeType}")
    lattice_res = goldLattice_111
    if latticeType == "gold-111":
        lattice_res = goldLattice_111
    elif latticeType == "gold-200":
        lattice_res = goldLattice_200
    elif latticeType == "graphitized-carbon":
        lattice_res = gc_res
    elif latticeType == "define_lattice_constant":
        lattice_res = latticeRes

    leeway_plus = 0.05
    leeway_minus = 0.04
    minradi = int((new_size[0]*(px_guess-leeway_minus))/lattice_res)
    maxradi = int((new_size[0]*(px_guess+leeway_plus))/lattice_res)
    min_radi_val = y[minradi]
    max_radi_val = y[maxradi]
    num_indices = 1+(maxradi-minradi)
    for i in range(minradi, maxradi+1): #now do the linear interpolation
        fraction = (i-minradi)/num_indices
        y[i] = min_radi_val - (fraction*(min_radi_val-max_radi_val))




    print("Smoothing radial profiles")
    print_text.append("Smoothing radial profiles")
    y2 = savgol_filter(y, savgol_window, 2)
    print("Radial profiles smoothed")
    print_text.append("Radial profiles smoothed")
    for i in range(len(y2)):
        if y2[i] < 0:
            y2[i] *= -1


    #now write to CSV
    new_list = zip(x, y)
    #write the results to a file
    f = open(path + "r-profile.csv", 'w')
    writer = csv.writer(f)
    writer.writerows(new_list)
    f.close()
    
    print("Making power spectrum")
    print_text.append("Making power spectrum")
    #also make a whole new mrc of the noise mrc
    write_mrc = loopImage(new_size, x, y2)


    #write to mrc
    print("Writing to file")
    print_text.append("Writing to file")
    
    with mrcfile.new(new_filename, overwrite=True) as mrc:
        mrc.set_data(np.float32(write_mrc))

    print("total time --- %s seconds ---" % round(time.time() - start_time, 2))
    print_text.append("total time --- %s seconds ---" % round(time.time() - start_time, 2))

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")






  



import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
import os
import sys
import mrcfile
import scipy
import csv
import math
from matplotlib.widgets import Slider
from matplotlib.patches import Ellipse
from scipy import ndimage
from scipy.signal import savgol_filter
from scipy.signal import find_peaks_cwt
from scipy.signal import find_peaks
from scipy.ndimage import gaussian_filter
from PIL import Image, ImageEnhance
from multiprocessing import Pool
from functools import partial


#this script is going to show a ring around the FFT so the user can check that this is sensible


anglemin = 0
anglemax = 180
savgol_window = 37
contrast_val = 10



coords = ''

def adjustContrast(data, c):
    thisQ = c*np.log10(1+data)
    return thisQ

def get_coords(x, y, angle, imwidth, imheight):
    endx1 = 0
    endx2 = imwidth
    endy1 = 0
    endy2 = imheight
    if angle == 0 or angle ==180:
        endy1 = imheight/2
        endy2 = imheight/2
    elif angle == 90 or angle ==270:
        endx1 = imwidth/2
        endx2 = imwidth/2
    else:
        x1_length = (x-imwidth) / math.cos(angle)
        y1_length = (y-imheight) / math.sin(angle)
        length = max(abs(x1_length), abs(y1_length))
        endx1 = x + length * math.cos(math.radians(angle))
        endy1 = y + length * math.sin(math.radians(angle))

        x2_length = (x-imwidth) / math.cos(angle+180)
        y2_length = (y-imheight) / math.sin(angle+180)
        length = max(abs(x2_length), abs(y2_length))
        endx2 = x + length * math.cos(math.radians(angle+180))
        endy2 = y + length * math.sin(math.radians(angle+180))

    return endx1, endy1, endx2, endy2

def getUserAngularRange(img, center, pxGuess, latticeRes, aliased, cores, fileSave):
    angles = (0, 180)
    blank_range = [int(center[0]/20), int(center[1]/20)]
    mod_img = img.copy()
    mod_img[int(center[0]-blank_range[0]):int(center[0]+blank_range[0]), int(center[1]-blank_range[1]):int(center[1]+blank_range[1])] = 0
    size = img.shape
    max_val = np.amax(mod_img)
    c = 255 / (np.log10(1+max_val))

    #vecAdjust = np.vectorize(adjustContrast)
    #Q = vecAdjust(mod_img, c)

    #split up image and process bits on separate cores for speed-up
    split_images = np.array_split(mod_img, cores)
    p = Pool(processes=cores)
    adjustContrast1 = partial(adjustContrast, c=c)
    image_dict = p.map(adjustContrast1, split_images)
    p.close()
    Q = np.concatenate(image_dict)
    
    data = np.int8(Q)
    img_show = Image.fromarray(np.uint8(data), 'L')
    enhancer = ImageEnhance.Contrast(img_show)
    enhanced = enhancer.enhance(contrast_val)
    #plot image
    fig = plt.figure()
    ax = fig.subplots()
    ax.set_xlabel(f"Pixel size: {round(pxGuess, 4)} $\AA$")
    image_object = plt.imshow(enhanced, cmap='gray')
    plt.subplots_adjust(bottom=0.2, left=0.10, right=0.95, top=0.90)
    ax_slide_contrast = plt.axes([0.05, 0.25, 0.0225, 0.63])
    contrast_slider = Slider(ax_slide_contrast, 'Contrast', valmin=1, valmax=100, valinit=contrast_val, valstep=1, orientation="vertical")


    #now plot the circle
    size = img.shape
    radius = int((size[0]*pxGuess)/latticeRes)
    size_alias =   2**(int(math.log(size[0], 2)))
    if aliased == 'y':
        radius = int((size_alias*pxGuess)/latticeRes)
    circle = Ellipse((size[0]/2, size[1]/2), 2*radius, 2*radius, color='r', fill=False, linestyle='--', linewidth=0.5)
    ax.add_artist(circle)
    

    def update_contrast(val):
        global contrast_val
        contrast_val = int(contrast_slider.val)
        enhanced = enhancer.enhance(contrast_val)
        image_object.set_data(enhanced)
        fig.canvas.draw()
    contrast_slider.on_changed(update_contrast)
    plt.savefig(fileSave + ".pdf")
    plt.show()
    return angles

def method2(img, pxGuess, latticeRes, aliased, cores, fileSave):
    #savgol_window = 41
    size = img.shape
    angles = (0, 180)
    px = 0
    #print(size[0])
    center = (size[0]/2, size[1]/2)
    #get the angles proper
    angles = getUserAngularRange(img, center, pxGuess, latticeRes, aliased, cores, fileSave)
    return px

def getLatticeRes(temperature, latticeType, latticeResPassed):
    gold_lattice_param = temperature*5.67075E-5 + 4.06111 #A
    #adjust for very low temperature
    if temperature < 43.0:
        gold_lattice_param = 43.0*5.67075E-5 + 4.06111 #A       
    goldLattice_111 = gold_lattice_param / (1**2+1**2+1**2)**0.5
    goldLattice_200 = gold_lattice_param / (2**2)**0.5
    #gc_res = 3.3378
    gc_res = 3.4187
    lattice_res = goldLattice_111
    if latticeType == "gold-111":
        lattice_res = goldLattice_111
    elif latticeType == "gold-200":
        lattice_res = goldLattice_200
    elif latticeType == "graphitized-carbon":
        lattice_res = gc_res
    elif latticeType == "define_lattice_constant":
        lattice_res = latticeResPassed
    return lattice_res

if __name__ == "__main__":
    path = sys.argv[1]
    fileSave = sys.argv[2]
    pxGuess = float(sys.argv[3])
    cores = int(sys.argv[4])
    latticeType = sys.argv[5]
    aliased = sys.argv[6]
    temperature = float(sys.argv[7])
    latticeResPassed = float(sys.argv[8])
    latticeRes = getLatticeRes(temperature, latticeType, latticeResPassed)
    px2 = []
    overall = []
    for filename in os.listdir(path):
        if filename.endswith(".mrc"):
            print(filename)
            mrc = mrcfile.open(path+filename)
            img = mrc.data
            mrc.close()
            pxsize = method2(img, pxGuess, latticeRes, aliased, cores, fileSave)

    




import sys
import mrcfile
import numpy as np
import math
import csv
import os
from multiprocessing import Pool
from functools import partial
import time
import psutil
from numba import jit, njit
from scipy.signal import savgol_filter
import matplotlib.pyplot as plt

savgol_window = 37

def adjustContrast(data, c):
    thisQ = c*np.log10(1+data)
    return thisQ

def radial_profile(data, center, angles):
    y,x = np.indices((data.shape)) # first determine radii of all pixels
    #yx = complex(y, x) #make complex
    all_angles = np.angle(x-center[0] + 1j*(y-center[1]), deg=True)+180 #get angles
    #excluded_indices = np.argwhere(all_angles < angles[0]) #find indices outside this angular range
    #to test

    r = np.sqrt((x-center[0])**2+(y-center[1])**2)
    ind = np.argsort(r.flat) # get sorted indices
    sr = r.flat[ind] # sorted radii

    sangles = all_angles.flat[ind] #sort angles in same way
    #included_indices = np.argwhere(sangles > angles[0])
    #print(angles[0])
    #print(angles[1])
    excluded_indices = np.argwhere(sangles < angles[0])
    excluded_indices2 = np.argwhere(sangles > angles[1])
     
    sim = data.flat[ind] # image values sorted by radii
    sim[excluded_indices] = 0 # image values sorted by radii
    sim[excluded_indices2] = 0 # image values sorted by radii
    ri = sr.astype(np.int32) # integer part of radii (bin size = 1)
    # determining distance between changes
    deltar = ri[1:] - ri[:-1] # assume all radii represented
    rind = np.where(deltar)[0] # location of changed radius
    nr = rind[1:] - rind[:-1] # number in radius bin
    csim = np.cumsum(sim, dtype=np.float64) # cumulative sum to figure out sums for each radii bin
    tbin = csim[rind[1:]] - csim[rind[:-1]] # sum for image values in radius bins
    radialprofile = tbin/nr # the answer
    #smoothed = savgol_filter(radialprofile, 99, 2)
    the_r = sr[rind]
    
    #return [abs(smoothed), the_r]
    return [abs(radialprofile), the_r]

def noiseWhiten(filename, data_path, new_dir, N_data):
    mrc = mrcfile.open(data_path + filename, permissive=True)
    data=mrc.data
    mrc.close()
    size = data.shape
    powerOfTwo_y = np.ceil(np.log2(size[0]))
    powerOfTwo_x = np.ceil(np.log2(size[1])) 
    powerOfTwo = powerOfTwo_y
    if powerOfTwo_x > powerOfTwo:
        powerOfTwo = powerOfTwo_x
    fft_width = int(2**powerOfTwo)
    power = np.absolute(np.fft.fftshift(np.fft.fft2(data, s=(fft_width, fft_width))))**2
    NWPS = power/N_data
    with mrcfile.new(f"{new_dir}/{filename[:-4]}_NWPS.mrc", overwrite=True) as mrc:
        mrc.set_data(np.float32(NWPS))

    return NWPS

@njit
def sum_Array(results):
    NWPS = np.zeros(results[0].shape)
    for val in results:
        NWPS += val
    return NWPS

def runScript(data_path, blank_path, cores, spectra_dir, write_dir, num_images):
    print_text = []
    #load blank file
    blank_file = [f for f in os.listdir(write_dir) if f.endswith(".mrc") and f.startswith("NPS_ra_")][0]
    NPS_file = f"{write_dir}{blank_file}"
    N_mrc =  mrcfile.open(NPS_file, permissive=True)
    N_data = N_mrc.data
    N_mrc.close()
    #make new directories
    new_dir = f"{write_dir}{spectra_dir}"
    if not os.path.exists(new_dir):
            os.mkdir(new_dir)
            os.mkdir(f"{new_dir}/avg")
            os.mkdir(f"{new_dir}/avg/10")
    file_list = [f for f in os.listdir(data_path) if (f.endswith(".mrc") and f.endswith("_PS.mrc") == False)]
    cryosparc = False
    for files in file_list:
        if "patch_aligned" in files or "rigid_aligned" in files:
            cryosparc = True
            break
    file_list2 = []
    if cryosparc == True:
        file_list2 = [f for f in file_list if ("patch_aligned" in f or "rigid_aligned" in f) and (f.endswith("aligned_doseweighted.mrc")==False)] 
        file_list = file_list2
    num_files = len(file_list)
    images = 0
    if num_images <= 0 or num_images > num_files:
        images = num_files
    else:
        images = num_images

    #update file_list
    file_list = file_list[:images]
    print("Beginning noise whitening")
    print_text.append("Beginning noise whitening")

    p = Pool(processes=cores)
    noiseWhiten1 = partial(noiseWhiten, data_path=data_path, new_dir=new_dir, N_data=N_data)
    results = []
    for i, NWPS in enumerate(p.imap_unordered(noiseWhiten1, file_list)):
        results.append(NWPS)
        print(f"Image {str(i+1)}/{str(images)} complete")
        print_text.append(f"Image {str(i+1)}/{str(images)} complete")
        if (i+1)>= images:
            break
    p.close()
    
    print("Saving sum image")
    print_text.append("Saving sum image")
    #summing results
    #this can actually cause an error if it's too large so split up a bit
    #get avialable memory
    '''
    results = np.array(results, dtype=np.float16)
    mem_avail = psutil.virtual_memory().available
    #mem_array = sys.getsizeof(results)
    mem_array = results.nbytes
    frac = mem_array/mem_avail
    print(frac)
    frac = frac+(frac/10)
    limit = 100
    if frac < 1:
        limit = images
    else:
        limit = int(images/frac)
    int_frac = int(frac)

    NWPS_sum = np.sum(results[:limit], axis=0)
    if int_frac >= 2:
        for i in range(int_frac-1):
            NWPS_sum += np.sum(results[limit*(i+1):limit*(i+2)], axis=0)
    if int_frac >= 1:
        NWPS_sum += np.sum(results[limit*int_frac:], axis=0)
    '''

    #alternative memory limit way
    NWPS_sum = sum_Array(results)


    '''
    for count, filename in enumerate(file_list):
        NWPS = noiseWhiten(filename, data_path, new_dir, N_data)
        print(f"Image {str(count+1)}/{str(images)} complete")

        if count == 0:
            NWPS_sum = NWPS.copy()
        else:
            NWPS_sum = np.add(NWPS_sum, NWPS)
    '''
    
    if images >= 1:
        with mrcfile.new(f"{new_dir}/avg/avgNWPS.mrc", overwrite=True) as mrc:
            mrc.set_data(np.float32(NWPS_sum))
    '''
    if images >= 20:
        print("Saving grouped sums")
        print_text.append("Saving sum image")
        #making sums so have 10 images
        num_to_avg = int((images/10))
        sum_arrays = np.add.reduceat(results, np.arange(0, images, num_to_avg), axis=0) #this averages to get 10
        for i, sumArray in enumerate(sum_arrays):
            with mrcfile.new(f"{new_dir}/avg/10/avgNWPS_{str(i+1)}.mrc", overwrite=True) as mrc:
                mrc.set_data(np.float32(sumArray))
    '''
    #again, have to do this in memory limited way, hex fine but not pcterm22
    if images >= 20:
        print("Saving grouped sums")
        print_text.append("Saving grouped sums")
        #making sums so have 10 images
        num_tot = 10
        num_to_avg = int(math.ceil(images/num_tot)) #num to average per image
        if (images/num_tot) < num_to_avg:
            num_tot -= 1
        for i in range(num_tot):
            start = i * num_to_avg
            end = (i+1) * num_to_avg
            if end <= len(results):
                sumArray = sum_Array(results[start:end])
                with mrcfile.new(f"{new_dir}/avg/10/avgNWPS_{str(i+1)}.mrc", overwrite=True) as mrc:
                    mrc.set_data(np.float32(sumArray))
                print(f"Saved {str(i+1)}/{str(num_tot)} images") 
                print_text.append(f"Saved {str(i+1)}/{str(num_tot)} images")    
    
    #Do some validation by saving NWPS scaled
    print("Calculating display NWPS")
    print_text.append("Calculating display NWPS")
    anglemin = 0
    anglemax = 360
    angles = (anglemin, anglemax)
    size = NWPS_sum.shape
    center = (size[0]/2, size[1]/2)
    blank_range = [int(center[0]/20), int(center[1]/20)]
    mod_img = NWPS_sum.copy()
    mod_img[int(center[0]-blank_range[0]):int(center[0]+blank_range[0]), int(center[1]-blank_range[1]):int(center[1]+blank_range[1])] = 0
    max_val = np.amax(mod_img)
    c = 255 / (np.log10(1+max_val))
    #split up image and process bits on separate cores for speed-up
    split_images = np.array_split(mod_img, cores)
    p = Pool(processes=cores)
    adjustContrast1 = partial(adjustContrast, c=c)
    image_dict = p.map(adjustContrast1, split_images)
    p.close()
    Q = np.concatenate(image_dict)
    data = np.int8(Q)
    if images >= 1:
        with mrcfile.new(f"{write_dir}avgNWPS_display.mrc", overwrite=True) as mrc:
            mrc.set_data(np.int8(data))

   #now also plot this
    radi = int(blank_range[0]*2)
    radi2 = int((size[0]/2)-1)
    result = radial_profile(NWPS_sum, center, angles)
    rad = result[0]
    smoothed = savgol_filter(rad, savgol_window, 2)
    ri = result[1]

    max_index = np.argmax(smoothed[radi:radi2])
    the_max = ri[max_index + radi]
    min_index = np.argmin(smoothed[radi:radi2])
    the_min = ri[min_index + radi]
    mm = 1/25.4
    fig = plt.figure()
    ax = fig.subplots()
    ax.set_ylim(smoothed[min_index+radi], smoothed[max_index+radi] + 0.02*smoothed[max_index+radi])
    p, = ax.plot(ri[radi:radi2], smoothed[radi:radi2], 'k')
    #p, = ax.plot(ri[radi:radi2], rad[radi:radi2], 'k')
    #plt.subplots_adjust(bottom=0.15, left=0.15, right=0.9, top=0.9)
    right_side = ax.spines["right"]
    right_side.set_visible(False)
    top_side = ax.spines["top"]
    top_side.set_visible(False)
    for axis in ['left', 'bottom']:
        ax.spines[axis].set_linewidth(1)
        ax.spines[axis].set_color([0,0,0,0.6])
    ax.tick_params(axis='both', which = 'major', labelsize=7, color=[0,0,0,0.6], width=1)
    ax.tick_params(axis='both', which = 'minor', labelsize=7, color=[0,0,0,0.6], width=1)
    plt.xticks(alpha=0.6)
    plt.yticks(alpha=0.6)
    ax.set_xlabel("Radius (pixels)", fontsize=7.0, alpha=0.6)
    ax.set_ylabel("Intensity", fontsize=7.0, alpha=0.6)
    plt.tight_layout()
    plt.savefig(f"{write_dir}avgNWPS_display_profile.pdf") 

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")
       
        
    
    
        
        
        
        
        
        




        



import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
import os
import sys
import mrcfile
import scipy
import csv
import pandas as pd
import math
import subprocess
import time
from matplotlib.widgets import Slider
from scipy import ndimage
from scipy.signal import savgol_filter
from scipy.signal import find_peaks_cwt
from scipy.signal import find_peaks
from scipy.ndimage import gaussian_filter
from PIL import Image, ImageEnhance
from multiprocessing import Pool
from functools import partial
from magCalibration.scripts import angles_choose
from magCalibration.scripts import smooth_choose
from lmfit.models import GaussianModel, VoigtModel, LinearModel, ConstantModel

#supervised_smooth = sys.argv[2]
#supervised_angle = sys.argv[3]


anglemin = 0
anglemax = 180
contrast_val = 50

cores = 8

coords = ''

def adjustContrast(data, c):
    thisQ = c*np.log10(1+data)
    return thisQ

def get_coords(x, y, angle, imwidth, imheight):
    endx1 = 0
    endx2 = imwidth
    endy1 = 0
    endy2 = imheight
    if angle == 0 or angle ==180:
        endy1 = imheight/2
        endy2 = imheight/2
    elif angle == 90 or angle ==270:
        endx1 = imwidth/2
        endx2 = imwidth/2
    else:
        x1_length = (x-imwidth) / math.cos(angle)
        y1_length = (y-imheight) / math.sin(angle)
        length = max(abs(x1_length), abs(y1_length))
        endx1 = x + length * math.cos(math.radians(angle))
        endy1 = y + length * math.sin(math.radians(angle))

        x2_length = (x-imwidth) / math.cos(angle+180)
        y2_length = (y-imheight) / math.sin(angle+180)
        length = max(abs(x2_length), abs(y2_length))
        endx2 = x + length * math.cos(math.radians(angle+180))
        endy2 = y + length * math.sin(math.radians(angle+180))

    return endx1, endy1, endx2, endy2

def getUserAngularRange(img, center):
    blank_range = [int(center[0]/20), int(center[1]/20)]
    mod_img = img.copy()
    mod_img[int(center[0]-blank_range[0]):int(center[0]+blank_range[0]), int(center[1]-blank_range[1]):int(center[1]+blank_range[1])] = 0
    size = img.shape
    max_val = np.amax(mod_img)
    c = 255 / (np.log10(1+max_val))

    #vecAdjust = np.vectorize(adjustContrast)
    #Q = vecAdjust(mod_img, c)

    #split up image and process bits on separate cores for speed-up
    split_images = np.array_split(mod_img, cores)
    p = Pool(processes=cores)
    adjustContrast1 = partial(adjustContrast, c=c)
    image_dict = p.map(adjustContrast1, split_images)
    p.close()
    Q = np.concatenate(image_dict)
    
    data = np.int8(Q)
    img_show = Image.fromarray(np.uint8(data), 'L')
    enhancer = ImageEnhance.Contrast(img_show)
    enhanced = enhancer.enhance(contrast_val)
    coords_angle = get_coords(center[0], center[1], 0, size[0], size[1])
    coords_angle2 = get_coords(center[0], center[1], 180, size[0], size[1])
    x = [coords_angle[0], coords_angle[2]]
    y = [coords_angle[1], coords_angle[3]]
    x2 = [coords_angle2[0], coords_angle2[2]]
    y2 = [coords_angle2[1], coords_angle2[3]]
    fig = plt.figure()
    ax = fig.subplots()
    p, = ax.plot(x, y, color="blue", linewidth=1, alpha=0.5)
    p2, = ax.plot(x2, y2, color="red", linewidth=1, alpha=0.5)
    #plt.imshow(data, cmap = 'gray')
    image_object = plt.imshow(enhanced, cmap = 'gray')
    plt.subplots_adjust(bottom=0.15, left=0.1)
    ax_slide = plt.axes([0.2, 0.05, 0.65, 0.03])
    win_size = Slider(ax_slide, 'Angle min', valmin=0, valmax=180, valinit=0, valstep=1, color='blue')
    ax_slide2 = plt.axes([0.2, 0.01, 0.65, 0.03])
    win_size2 = Slider(ax_slide2, 'Angle max', valmin=0, valmax=180, valinit=180, valstep=1, color='red')
    ax_slide_contrast = plt.axes([0.05, 0.25, 0.0225, 0.63])
    contrast_slider = Slider(ax_slide_contrast, 'Contrast', valmin=1, valmax=100, valinit=contrast_val, valstep=1, orientation="vertical")

    def on_press(event):
        sys.stdout.flush()
        global coords
        coords = event.key
        fig.canvas.mpl_disconnect(cid)
        plt.close(fig)
        return coords
    def update(val):
        global anglemin, anglemax
        anglemin = int(win_size.val)
        anglemax = int(win_size2.val)
        #print(anglemin)
        #print(anglemax)
        if anglemax < anglemin:
            win_size2.set_val(win_size.val+1)
            anglemax = anglemin+1
        coords_angle = get_coords(center[0], center[1], anglemin, size[0], size[1])
        coords_angle2 = get_coords(center[0], center[1], anglemax, size[0], size[1])
        x = [coords_angle[0], coords_angle[2]]
        y = [coords_angle[1], coords_angle[3]]
        x2 = [coords_angle2[0], coords_angle2[2]]
        y2 = [coords_angle2[1], coords_angle2[3]]
        #plt.clf()
        p.set_data(x,y)
        p2.set_data(x2,y2)
        fig.canvas.draw()
    def update_contrast(val):
        global contrast_val
        contrast_val = int(contrast_slider.val)
        enhanced = enhancer.enhance(contrast_val)
        image_object.set_data(enhanced)
        fig.canvas.draw()
    win_size.on_changed(update)
    win_size2.on_changed(update)
    contrast_slider.on_changed(update_contrast)
    cid = fig.canvas.mpl_connect('key_press_event', on_press)
    plt.show()
    
    angles = (anglemin, anglemax)
    #print(anglemin)
    #print(anglemax)
    return angles

def radial_profile(data, center, angles):
    y,x = np.indices((data.shape)) # first determine radii of all pixels
    #yx = complex(y, x) #make complex
    all_angles = np.angle(x-center[0] + 1j*(y-center[1]), deg=True)+180 #get angles
    #excluded_indices = np.argwhere(all_angles < angles[0]) #find indices outside this angular range
    #to test

    r = np.sqrt((x-center[0])**2+(y-center[1])**2)
    ind = np.argsort(r.flat) # get sorted indices
    sr = r.flat[ind] # sorted radii

    sangles = all_angles.flat[ind] #sort angles in same way
    #included_indices = np.argwhere(sangles > angles[0])
    #print(angles[0])
    #print(angles[1])
    excluded_indices = np.argwhere(sangles < int(angles[0]))
    excluded_indices2 = np.argwhere(sangles > int(angles[1]))
     
    sim = data.flat[ind] # image values sorted by radii
    sim[excluded_indices] = 0 # image values sorted by radii
    sim[excluded_indices2] = 0 # image values sorted by radii
    ri = sr.astype(np.int32) # integer part of radii (bin size = 1)
    # determining distance between changes
    deltar = ri[1:] - ri[:-1] # assume all radii represented
    rind = np.where(deltar)[0] # location of changed radius
    nr = rind[1:] - rind[:-1] # number in radius bin
    csim = np.cumsum(sim, dtype=np.float64) # cumulative sum to figure out sums for each radii bin
    tbin = csim[rind[1:]] - csim[rind[:-1]] # sum for image values in radius bins
    radialprofile = tbin/nr # the answer
    #smoothed = savgol_filter(radialprofile, 99, 2)
    the_r = sr[rind]
    
    #return [abs(smoothed), the_r]
    return [abs(radialprofile), the_r]

def method2(img, lattice_res, pxGuess, savgol_window, supervised_angle, NWPS_dir, filename, write_dir, supervised_smooth, aliased):
    size = img.shape
    str_alias = 'n'
    leeway_plus = 0.05
    leeway_minus = 0.04
    anglemin = 0
    anglemax = 180
    angles = (anglemin, anglemax)
    #the size must be converted back to power of 2 if aliased
    size_alias =   2**(int(math.log(size[0], 2)))  
    minradi = int((size[0]*(pxGuess-leeway_minus))/lattice_res)
    maxradi = int((size[0]*(pxGuess+leeway_plus))/lattice_res)
    if aliased:
        str_alias = 'y'
        minradi = int((size_alias*(pxGuess-leeway_minus))/lattice_res)
        maxradi = int((size_alias*(pxGuess+leeway_plus))/lattice_res)   
    center, radi, radi2 = (size[0]/2, size[1]/2), minradi, maxradi
    #get the angles proper
    accept = ""
    reject = ""
    end = "False"
    if (supervised_angle == True):
        #angles = getUserAngularRange(img, center)
        myMatProcess = subprocess.Popen([sys.executable, '-m', 'magCalibration.scripts.angles_choose', NWPS_dir, filename, str(cores), write_dir])
        time_to_wait = 999
        time_counter = 0
        while not os.path.exists(f"{write_dir}user_angles_temp.csv"):
            time.sleep(1)
            time_counter += 1
            if time_counter > time_to_wait:break
        #now get the angles
        with open(f"{write_dir}user_angles_temp.csv", 'r') as f:
            line = f.readline()
            l = line.split(',')
            anglemin = int(l[0])
            anglemax = int(l[1])
            angles = (anglemin, anglemax)
            accept = l[2]
            reject = l[3]
            end = l[4].strip("\n")
        #now delete the file
        os.remove(f"{write_dir}user_angles_temp.csv")
    else:
        accept = "True"
        reject = "False"
        end = "False"

    max_index = 0
    px = 0
    if accept == "True" and end != "True":
        if supervised_smooth != True:
            result = radial_profile(img, center, angles)
            rad = result[0]
            smoothed = savgol_filter(rad, savgol_window, 2)
            ri = result[1]

            max_index = np.argmax(smoothed[radi:radi2])
            the_max = ri[max_index + radi]
            min_index = np.argmin(smoothed[radi:radi2])
            the_min = ri[min_index + radi]

            px = (the_max/(size[0]))*lattice_res
            if aliased:
                px = (the_max/size_alias)*lattice_res

            #Here I should try and get a better localisation by doing some fitting (Voigt plus linear)
            model = VoigtModel() + LinearModel()
            params = model.make_params(amplitude=1, center=the_max, sigma=1, gamma=1, slope=0, intercept=1)
            #update the center bounds
            radi = int((size_alias*(px-leeway_minus))/lattice_res)
            radi2 = int((size_alias*(px+leeway_plus))/lattice_res) 
            #bounds
            params['center'].min = ri[radi]
            params['center'].max = ri[radi2]
            y = smoothed[radi:radi2]
            x = ri[radi:radi2]
            fit_result = model.fit(y, params, x=x)
            cen_r = fit_result.params['center'].value
            new_px = (cen_r/(size[0]))*lattice_res
            if aliased:
                new_px = (cen_r/size_alias)*lattice_res  

            #only accept if px change is less than a pixel
            if abs(cen_r - the_max) < 1.0:
                px = new_px  
 
            #Look at the fits manually to start with
            xx = np.linspace(ri[radi], ri[radi2], 100)
            yy = fit_result.eval(x=xx)
            #plt.plot(xx, yy, 'k')
            #plt.plot(x, y, 'bx', markersize=1)
            #plt.savefig(f"{write_dir}Voigt_fit.png")




            #I'll need to print out some of the fitting parameters and then average as well as just the centre. This can let me subtract better and maybe refine with a better background subtraction
            '''
            #Now enter the refinement by fitting the hcp peak, have to assume you have the correct peak
            model2 = VoigtModel() + LinearModel()
            params2 = model2.make_params(amplitude=1, center=cen_r-cen_r*0.05, sigma=1, gamma=1, slope=0, intercept=1)
            #update the center bounds
            radi21 = int((size_alias*(px-px*0.08))/lattice_res)
            radi22 = int((size_alias*(px-px*0.02))/lattice_res)
            #bounds
            params2['center'].min = ri[radi21]
            params2['center'].max = ri[radi22]
            y2 = smoothed[radi21:radi22]
            x2 = ri[radi21:radi22]
            fit_result2 = model2.fit(y2, params2, x=x2)
            yy2 = fit_result2.eval(x=x)
            
            smoothed_mod = y - yy2
            model3 = VoigtModel() + LinearModel()
            params3 = model3.make_params(amplitude=1, center=cen_r, sigma=1, gamma=1, slope=0, intercept=1)
            #bounds
            params3['center'].min = ri[radi]
            params3['center'].max = ri[radi2]
            y3 = smoothed_mod
            fit_result3 = model3.fit(y3, params3, x=x)
            cen_r = fit_result3.params['center'].value
            px = (cen_r/(size[0]))*lattice_res
            if aliased:
                px = (cen_r/size_alias)*lattice_res 
            '''
        else: #supervise px choise
            myMatProcess = subprocess.Popen([sys.executable, '-m', 'magCalibration.scripts.smooth_choose', NWPS_dir, filename, write_dir, str(lattice_res), str(pxGuess), str(anglemin), str(anglemax), str_alias])
            time_to_wait = 999
            time_counter = 0
            while not os.path.exists(f"{write_dir}user_px_temp.csv"):
                time.sleep(1)
                time_counter += 1
                if time_counter > time_to_wait:break
            #now get the px
            with open(f"{write_dir}user_px_temp.csv", 'r') as f:
                line = f.readline()
                l = line.split(',')
                px = float(l[0])
                end = l[1].strip("\n")
            #now delete the file
            os.remove(f"{write_dir}user_px_temp.csv")
            max_index = 999
    else:
        px = 0
    
    if (max_index <= 1):
        px = 0
    #print(px)
    return [px, end]

def runScript(path, pxGuess, temperature, cores, savgol_window, supervised_angle, aliased, aniso_corrected, spectra_dir, write_dir, ps_dir, latticeType, supervised_smooth, avgImage, latticeRes):
    print_text = []    
    gold_lattice_param = temperature*5.67075E-5 + 4.06111 #A
    #adjust for very low temperature
    if temperature < 43.0:
        gold_lattice_param = 43.0*5.67075E-5 + 4.06111 #A       
    goldLattice_111 = gold_lattice_param / (1**2+1**2+1**2)**0.5
    goldLattice_200 = gold_lattice_param / (2**2)**0.5
    #gc_res = 3.3378
    gc_res = 3.4187
    print(f"Lattice type is {latticeType}")
    print_text.append(f"Lattice type is {latticeType}")
    lattice_res = goldLattice_111
    if latticeType == "gold-111":
        lattice_res = goldLattice_111
    elif latticeType == "gold-200":
        lattice_res = goldLattice_200
    elif latticeType == "graphitized-carbon":
        lattice_res = gc_res
    elif latticeType == "define_lattice_constant":
        lattice_res = latticeRes

    overall = []
    file_list = []
    #NWPS_dir = f"{path}{spectra_dir}/avg/10/"
    NWPS_dir = ps_dir
    '''
    if aniso_corrected:
        NWPS_dir = NWPS_dir + "stretch/"
    if aliased == False:
        file_list = [f for f in os.listdir(NWPS_dir) if (f.endswith(".mrc") and f.endswith("aliased.mrc") == False)]
    else:
        file_list = [f for f in os.listdir(NWPS_dir) if f.endswith("aliased.mrc")]
    num_files = len(file_list)

    if num_files < 1:
        NWPS_dir = f"{path}{spectra_dir}/avg/"
        if aniso_corrected:
            NWPS_dir = NWPS_dir + "stretch/"
        file_list = [f for f in os.listdir(NWPS_dir) if f.endswith(".mrc")]
        if aliased == False:
            file_list = [f for f in os.listdir(NWPS_dir) if (f.endswith(".mrc") and f.endswith("aliased.mrc") == False)]
        else:
            file_list = [f for f in os.listdir(NWPS_dir) if f.endswith("aliased.mrc")]
        num_files = len(file_list)
    '''
    file_list = [f for f in os.listdir(NWPS_dir) if f.endswith(".mrc")]
    num_files = len(file_list)
    if num_files == 1:
        print(f"Calculating pixel size for the averaged FFT")
        print_text.append(f"Calculating pixel size for the averaged FFT")
    else:
        print(f"Calculating pixel sizes for individual FFTs")
        print_text.append(f"Calculating pixel sizes for individual FFTs")          
    #print(NWPS_dir)
    #print(file_list)
    #file_list = [f for f in os.listdir(path) if f.endswith(".mrc")]
    for i, filename in enumerate(file_list):
        
        mrc = mrcfile.open(NWPS_dir+filename)
        img = mrc.data
        mrc.close() 
        result = method2(img, lattice_res, pxGuess, savgol_window, supervised_angle, NWPS_dir, filename, write_dir, supervised_smooth, aliased)
        pxsize = result[0]
        end = result[1]
        if end == "True":
            break
        if pxsize > 0 and end != "True":
            overall.append([filename, pxsize])
            print(f"Px size for image {i+1}/{num_files} calculated")
            print_text.append(f"Px size for image {i+1}/{num_files} calculated")
            if num_files == 1: #then this is the average image or image on its owned
                print(f"Px size for sum FFT is: {str(round(pxsize, 4))}")
                print_text.append(f"Px size for sum FFT is: {str(round(pxsize, 4))}")        
        #check if user wants to continue if supervised
    #f = open(NWPS_dir + "pxSizes_smooth" + str(savgol_window) + ".csv", 'w')
    write_pxFile = f"{write_dir}pxSizesAvgImage.csv"
    if avgImage == False:
        write_pxFile = f"{write_dir}pxSizes.csv"
    with open(write_pxFile, 'w') as f:
        writer = csv.writer(f)
        writer.writerows(overall)
    #f.close()  
    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")   

    




import sys
import mrcfile
import numpy as np
import math
import csv
import os
from matplotlib.widgets import Slider
from scipy.signal import savgol_filter
import matplotlib.pyplot as plt
from numba import jit, njit
from multiprocessing import Pool
from functools import partial
import time
from PIL import Image, ImageEnhance
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure

#path = sys.argv[1]
savgol_window = 91
contrast_val = 2
coords = ''
anglemin=0
anglemax=180

def getAngles(write_dir):
    angles = []
    the_files = []
    with open(f"{write_dir}user_angles.csv", 'r') as f:
        for line in f.readlines():
            l = line.split(',')
            if len(l) > 1:
                angles.append([float(l[0]), float(l[1])])
                the_file = l[2].rstrip("\n")
                the_files.append(the_file)

    return [angles, the_files]

def getBoxes(write_dir):
    boxes = []
    the_files = []
    fractions = []
    with open(f"{write_dir}user_boxes.csv", 'r') as f:
        for line in f.readlines():
            l = line.split(',')
            if len(l) > 2:
                boxes.append([int(float(l[0])), int(float(l[1])), int(float(l[2])), int(float(l[3]))])
                the_file = l[4].rstrip("\n")
                the_files.append(the_file)
                fraction = l[5].rstrip("\n")
                fractions.append(float(fraction))
    return [boxes, the_files, fractions]

@njit
def loopImage(size, x, y):
    
    #make an array of ones of same size as image
    write_mrc = np.ones(size)
    #print(size)
    centre = (int(size[0]/2), int(size[1]/2))
    max_distance = np.max(x)
    #go through each pixel and get the distance to centre
    for h in range(size[0]):
        for w in range(size[1]):
            #get distance to centre
            distance = ((centre[0]-h)**2 + (centre[1]-w)**2)**0.5
            #interpolate this value in profile if it can be done
            if distance <= max_distance:
                write_mrc[h][w] = np.interp(distance, x, y)
            else:
                write_mrc[h][w] = np.interp(max_distance, x, y)
    
    return write_mrc
                

def radial_profile(data, center, angles, fraction_box):
    fraction_angle = (angles[1]-angles[0])/360
    y,x = np.indices((data.shape)) # first determine radii of all pixels
    #yx = complex(y, x) #make complex
    all_angles = np.angle(x-center[0] + 1j*(y-center[1]), deg=True)+180 #get angles
    #excluded_indices = np.argwhere(all_angles < angles[0]) #find indices outside this angular range
    #to test

    r = np.sqrt((x-center[0])**2+(y-center[1])**2)
    ind = np.argsort(r.flat) # get sorted indices
    sr = r.flat[ind] # sorted radii

    sangles = all_angles.flat[ind] #sort angles in same way
    #included_indices = np.argwhere(sangles > angles[0])
    #print(angles[0])
    #print(angles[1])
    excluded_indices = np.argwhere(sangles < angles[0])
    excluded_indices2 = np.argwhere(sangles > angles[1])
     
    sim = data.flat[ind] # image values sorted by radii
    sim[excluded_indices] = 0 # image values sorted by radii
    sim[excluded_indices2] = 0 # image values sorted by radii
    ri = sr.astype(np.int32) # integer part of radii (bin size = 1)
    # determining distance between changes
    deltar = ri[1:] - ri[:-1] # assume all radii represented
    rind = np.where(deltar)[0] # location of changed radius
    nr = rind[1:] - rind[:-1] # number in radius bin
    csim = np.cumsum(sim, dtype=np.float64) # cumulative sum to figure out sums for each radii bin
    tbin = csim[rind[1:]] - csim[rind[:-1]] # sum for image values in radius bins
    radialprofile = tbin/nr # the answer
    #radialprofile /= fraction_angle #adjust for smaller angular range
    # radialprofile /= fraction_box #adjust for cropped box #currently removed as it will upweight smaller images
    #smoothed = savgol_filter(radialprofile, 99, 2)
    the_r = sr[rind]
    
    #return [abs(smoothed), the_r]
    return [abs(radialprofile), the_r]


def runScript(path, cores, doAngles, cropImage, write_dir):
    new_filename = ""
    if doAngles == True and cropImage == False:
        new_filename = f"{write_dir}NPS_ra_angle_smooth{str(savgol_window)}.mrc"
    elif doAngles == False and cropImage == True:
        new_filename = f"{write_dir}NPS_ra_crop_smooth{str(savgol_window)}.mrc"
    elif doAngles == True and cropImage == True:
        new_filename = f"{write_dir}NPS_ra_crop-angle_smooth{str(savgol_window)}.mrc"
    if not os.path.exists(new_filename):
        theRealProcessing(path, cores, new_filename, doAngles, cropImage, write_dir)


def theRealProcessing(path, cores, new_filename, doAngles, cropImage, write_dir):
    print_text = []
    print("Making noise power spectrum from given images")
    print_text.append("Making noise power spectrum from given images")
    #First get 
    angles = []
    boxes = []
    file_list = []
    fractions = []  
    if doAngles:
        angle_data = getAngles(write_dir)
        angles = angle_data[0]
        file_list = angle_data[1]
    if cropImage:
        box_data = getBoxes(write_dir)
        boxes = box_data[0]
        file_list = box_data[1]
        fractions = box_data[2]
    noise_data = []
    print("Loading images")
    print_text.append("Loading images")
    for filename in file_list:
        mrc_blank = mrcfile.open(path + filename, permissive=True)
        noise_data.append(mrc_blank.data)
        mrc_blank.close()
    print("Images loaded")
    print_text.append("Images loaded")
    size = noise_data[0].shape
    powerOfTwo_y = np.ceil(np.log2(size[0]))
    powerOfTwo_x = np.ceil(np.log2(size[1])) 
    powerOfTwo = powerOfTwo_y
    if powerOfTwo_x > powerOfTwo:
        powerOfTwo = powerOfTwo_x
    fft_width = int(2**powerOfTwo)
    new_size = (fft_width, fft_width)
    noise_power = []
    count = len(noise_data)
    x = []
    y = []


    #Now I should crop if cropIMage
    if cropImage:
        for i, this_data in enumerate(noise_data):
            box_coords = boxes[i]
            arr = np.array(this_data)
            noise_data[i] = this_data[box_coords[2]:box_coords[3],box_coords[0]:box_coords[1]]
            if doAngles == False:
                angles.append([0, 180])


    start_time = time.time()

    print("Calculating radial profile of FFTs...")
    print_text.append("Calculating radial profile of FFTs...")
    for i in range(count):
        noise_power = np.absolute(np.fft.fftshift(np.fft.fft2(noise_data[i], s=(int(fft_width), int(fft_width)))))**2
        #now I want to do the radial profile
        size = noise_power.shape
        center = (size[0]/2, size[1]/2)
        result = radial_profile(noise_power, center, angles[i], fractions[i])
        rad = result[0]
        #smoothed = savgol_filter(rad, savgol_window, 2)
        ri = result[1][:-1]
        if i == 0:
            x = ri
            y = rad.copy()
        else:
            y = np.add(y, rad)
        print(f"Finished image {str(i+1)}/{count}") 
        print_text.append(f"Finished image {str(i+1)}/{count}")
        
    mean_fraction = np.mean(fractions)
    y /= count #average
    #y /= mean_fraction # upweight for smaller image sizes
    
    print("Power spectra radial profiles calculated --- %s seconds ---" % round(time.time() - start_time,2))
    print_text.append("Power spectra radial profiles calculated --- %s seconds ---" % round(time.time() - start_time,2))
    print("Smoothing radial profiles")
    print_text.append("Smoothing radial profiles")
    y2 = savgol_filter(y, savgol_window, 2)
    print("Radial profiles smoothed")
    print_text.append("Radial profiles smoothed")
    for i in range(len(y2)):
        if y2[i] < 0:
            y2[i] *= -1

    #now plot
    '''
    plt.plot(x, y)
    plt.ylim(1E16)
    plt.show()
    '''
    
    #now write to CSV
    new_list = zip(x, y)
    #write the results to a file
    f = open(write_dir + "r-profile.csv", 'w')
    writer = csv.writer(f)
    writer.writerows(new_list)
    f.close()
    
    #also make a whole new mrc of the noise mrc
    write_mrc = loopImage(new_size, x, y2)

    #write to mrc
    print("Writing to file")
    print_text.append("Writing to file")
    
    with mrcfile.new(new_filename, overwrite=True) as mrc:
        mrc.set_data(np.float32(write_mrc))

    print("total time --- %s seconds ---" % round(time.time() - start_time, 2))
    print_text.append("total time --- %s seconds ---" % round(time.time() - start_time, 2))

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")





  



import cv2
import PyQt5
from PyQt5.QtCore import QLibraryInfo
import sys
import mrcfile
import numpy as np
import math
import csv
import os
from pathlib import Path
from matplotlib.widgets import Slider
from matplotlib.widgets import TextBox
from matplotlib.widgets import Button
from matplotlib.patches import Ellipse
from matplotlib.patches import Rectangle
from matplotlib.widgets import RangeSlider
import matplotlib.pyplot as plt
import matplotlib as mpl
from PIL import Image, ImageEnhance
import skimage.io
import skimage.color
import skimage.filters
from numba import jit, njit
from skimage.measure import block_reduce

os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = QLibraryInfo.location(QLibraryInfo.PluginsPath)

left_val = 0
right_val = 0
top_val = 0
bottom_val = 0

low_hough = 140
up_hough = 200

minVal = 0
maxVal = 999

x = 0
y = 0
rx = 0
ry = 0
square_x = 0
square_y = 0
square_rx = 0
square_ry = 0

accept = False
reject = False
end = False

output = []

circle2 = Ellipse((2048, 2048), 500, 500, color='r', fill=False)
square2 = Rectangle((2048, 2048), 500, 500, color='g', fill=False)
centre_square = Rectangle((2038, 2038), 20, 20, color='k', fill=False)

contrast_val = 2

def cropImage(data, filename):
    global left_val, right_val, top_val, bottom_val
    size = data.shape
    #print(size)
    fig = plt.figure()
    ax = fig.subplots()
    left_val = 0
    right_val = size[1]
    top_val = 0
    bottom_val = size[0]

    left_x = [0, 0]
    left_y = [0, size[0]]
    right_x = [size[1], size[1]]
    right_y = [0, size[0]]
    top_x = [0, size[1]]
    top_y = [0, 0]
    bottom_x = [0, size[1]]
    bottom_y = [size[0], size[0]]
    p_left, = ax.plot(left_x, left_y, color="red", linewidth=1, alpha=0.5)
    p_right, = ax.plot(right_x, right_y, color="red", linewidth=1, alpha=0.5)
    p_top, = ax.plot(top_x, top_y, color="red", linewidth=1, alpha=0.5)
    p_bottom, = ax.plot(bottom_x, bottom_y, color="red", linewidth=1, alpha=0.5)

    normal_array = data/np.amax(data)
    img = Image.fromarray(np.uint8(normal_array*255), 'L')
    enhancer = ImageEnhance.Contrast(img)
    enhanced = enhancer.enhance(2)
    plt.imshow(enhanced, cmap = 'gray')

    plt.subplots_adjust(bottom=0.15, left=0.1, right=0.9, top=0.9)
    ax_bottom = plt.axes([0.15, 0.05, 0.65, 0.03])
    bottom_slider = Slider(ax_bottom, 'Bottom', valmin=0, valmax=size[0]-1, valinit=size[0]-1, valstep=2)
    ax_top = plt.axes([0.15, 0.95, 0.65, 0.03])
    top_slider = Slider(ax_top, 'Top', valmin=0, valmax=size[0]-1, valinit=0, valstep=2)
    ax_left = plt.axes([0.05, 0.25, 0.0225, 0.63])
    left_slider = Slider(ax_left, 'Left', valmin=0, valmax=size[1]-1, valinit=0, valstep=2, orientation="vertical")
    ax_right = plt.axes([0.9, 0.25, 0.0225, 0.63])
    right_slider = Slider(ax_right, 'Right', valmin=0, valmax=size[1]-1, valinit=size[1]-1, valstep=2, orientation="vertical")

    def on_press(event):
        sys.stdout.flush()
        global coords
        coords = event.key
        fig.canvas.mpl_disconnect(cid)
        plt.close(fig)
        return coords
    def update(val):
        global left_val, right_val, top_val, bottom_val
        left_val = int(left_slider.val)
        right_val = int(right_slider.val)
        top_val = int(top_slider.val)
        bottom_val = int(bottom_slider.val)
        #print(anglemin)
        #print(anglemax)
        if right_val < left_val:
            right_slider.set_val(left_slider.val+1)
            right_val = left_val+1
        if bottom_val < top_val:
            bottom_slider.set_val(top_slider.val+1)
            bottom_val = top_val+1
        left_x = [left_val, left_val]
        left_y = [0, size[0]]
        right_x = [right_val, right_val]
        right_y = [0, size[0]]
        top_x = [0, size[1]]
        top_y = [top_val, top_val]
        bottom_x = [0, size[1]]
        bottom_y = [bottom_val, bottom_val]
        #plt.clf()
        p_left.set_data(left_x,left_y)
        p_right.set_data(right_x,right_y)
        p_top.set_data(top_x,top_y)
        p_bottom.set_data(bottom_x,bottom_y)
        fig.canvas.draw()
    bottom_slider.on_changed(update)
    top_slider.on_changed(update)
    left_slider.on_changed(update)
    right_slider.on_changed(update)
    cid = fig.canvas.mpl_connect('key_press_event', on_press)
    plt.show()
    return [int(left_val), int(right_val), int(top_val), int(bottom_val), filename]

def cropImage2(data, filename):
    global x, y, rx, ry, circle2, square2, square_x, square_y, square_rx, square_ry, centre_square, accept, reject, end, low_hough, up_hough, minVal, maxVal
    accept = False
    reject = False
    end = False
    size = data.shape
    #Blur the image
    binFactor = 4
    blurred_image = skimage.filters.gaussian(data, sigma=1.0)
    blurred_image = block_reduce(blurred_image, block_size=(binFactor, binFactor), func=np.max)
    t = skimage.filters.threshold_otsu(blurred_image)
    #binary_mask = blurred_image > t
    maxVal = np.max(blurred_image)
    data3 = blurred_image/maxVal
    data3 = 255 * data3
    img = data3.astype(np.uint8) 
    wide = cv2.Canny(img, low_hough, up_hough)
    circles = cv2.HoughCircles(wide, cv2.HOUGH_GRADIENT, 3, int(size[0]/(2*binFactor)))

    #output = block_reduce(blurred_image, block_size=(binFactor, binFactor), func=np.max) #blurred_image.copy()
    output = blurred_image.copy()
    # ensure at least some circles were found
    x = int(size[1]/(2*binFactor))
    y = int(size[0]/(2*binFactor))
    r = int(size[0]/(4*binFactor))
    if circles is not None:
        # convert the (x, y) coordinates and radius of the circles to integers
        circles = np.round(circles[0, :]).astype("int")
        # loop over the (x, y) coordinates and radius of the circles
        (x,y,r) = circles[0]
        #cv2.circle(output, (x, y), r-(int(r/10)), (0, 0, 255), 4)
        #cv2.rectangle(output, (x - 5, y - 5), (x + 5, y + 5), (0, 128, 255), -1)

        # show the output image

    #image contrast
    #img_show = Image.fromarray(np.uint8(output), 'L')
    #enhancer = ImageEnhance.Contrast(img_show)
    #enhanced = enhancer.enhance(contrast_val) 
    if np.min(output) < 0:
        output -= np.min(output)
    output = output/np.max(output)
    output *= 255
    output_int8 = output.astype(np.uint8)
    contrast_output = np.copy(output_int8)
    minVal = np.min(output_int8)
    maxVal = np.max(output_int8)
    contrast_output[output_int8 < minVal] = 0
    contrast_output[output_int8 > maxVal] = 255
    #output = (output < maxVal) * output
    #output = (output > minVal) * output

    fig = plt.figure()
    ax = fig.subplots()
    plt.subplots_adjust(bottom=0.2, left=0.10, right=0.95, top=0.90)
    #plt.imshow(enhanced, cmap='gray')
    image_object = plt.imshow(contrast_output, cmap = 'gray')
    rx = r-(int(r/10))
    ry = r-(int(r/10))
    #circle2 = plt.Circle((x, y), r, color='r', fill=False)
    circle2 = Ellipse((x, y), 2*rx, 2*ry, color='r', fill=False)
    #circle_patch, = ax.add_patch(circle2)
    ax.add_artist(circle2)
    #now work out and plot square, the box coords, later change to ellipse
    square_rx = (rx**2 / 2)**0.5
    square_ry = (ry**2 / 2)**0.5
    square_x = x - square_rx
    square_y = y - square_ry
    square2 = Rectangle((square_x, square_y), square_rx*2, square_ry*2, color='g', fill=False)
    ax.add_artist(square2)
    centre_square = Rectangle((x - 10, y - 10), 20, 20, color='k', fill=True)
    ax.add_artist(centre_square)
    #sqaure_patch, = ax.add_patch(square2)

    axbox_x = fig.add_axes([0.2, 0.10, 0.25, 0.03]) #left, bottom, width, height
    text_box_x = TextBox(axbox_x, "centre x:", initial=str(x))

    axbox_y = fig.add_axes([0.6, 0.10, 0.25, 0.03])
    text_box_y = TextBox(axbox_y, "centre y:", initial=str(y))

    axbox_rx = fig.add_axes([0.2, 0.05, 0.25, 0.03]) #left, bottom, width, height
    text_box_rx = TextBox(axbox_rx, "radius x:", initial=str(rx))

    axbox_ry = fig.add_axes([0.6, 0.05, 0.25, 0.03]) #left, bottom, width, height
    text_box_ry = TextBox(axbox_ry, "radius y:", initial=str(ry))

    axAccept = plt.axes([0.1, 0.93, 0.2, 0.05])
    bAccept = Button(axAccept, 'Accept')
    axReject = plt.axes([0.4, 0.93, 0.2, 0.05])
    bReject = Button(axReject, 'Reject')
    axEnd = plt.axes([0.7, 0.93, 0.2, 0.05])
    bEnd = Button(axEnd, 'End')

    #ax_left = plt.axes([0.05, 0.25, 0.0225, 0.63])
    #left_slider = RangeSlider(ax_left, 'Left_threshold', valmin=0, valmax=250, valinit=(low_hough, up_hough), valstep=1, orientation="vertical")
    ax_slide_contrast = plt.axes([0.05, 0.25, 0.0225, 0.63])
    contrast_slider = RangeSlider(ax_slide_contrast, 'Left_threshold', valmin=minVal, valmax=maxVal, valinit=(minVal, maxVal), valstep=1, orientation="vertical")  

    def submitx(expression):
        global x, y, circle2, square2, square_rx, square_x, centre_square
        x = int(expression)
        square_x = x - square_rx
        #print(x)
        circle2.set_center((x, y))
        square2.set_x(square_x)
        centre_square.set_x(x - 10)
        fig.canvas.draw()

    def submity(expression):
        global x, y, circle2, square2, square_ry, square_y, centre_square
        y = int(expression)
        square_y = y - square_ry
        #print(x)
        circle2.set_center((x, y))
        square2.set_y(square_y)
        centre_square.set_y(y - 10)
        fig.canvas.draw()

    def onclickSquare(event):
        global x, y, circle2, square2, square_ry, square_y, centre_square
        temp_x, temp_y = int(event.xdata), int(event.ydata)
        if temp_x > 0 and temp_y > 0 :
            x = temp_x
            y = temp_y
            square_x = x - square_rx
            square_y = y - square_ry
            circle2.set_center((x, y))
            square2.set_x(square_x)
            square2.set_y(square_y)
            centre_square.set_x(x - 10)
            centre_square.set_y(y - 10)
            fig.canvas.draw()

    def submitrx(expression):
        global x, y, circle2, square2, square_rx, square_x, rx
        rx = int(expression)
        square_rx = (rx**2 / 2)**0.5
        square_x = x - square_rx
        #print(x)
        circle2.width = 2*rx
        square2.set_x(square_x)
        square2.set_width(square_rx*2)
        fig.canvas.draw()

    def submitry(expression):
        global x, y, circle2, square2, square_ry, square_y, ry
        ry = int(expression)
        square_ry = (ry**2 / 2)**0.5
        square_y = y - square_ry
        #print(x)
        circle2.height = 2*ry
        square2.set_y(square_y)
        square2.set_height(square_ry*2)
        fig.canvas.draw()

    def update_contrast(val):
        global minVal, maxVal
        minVal = val[0]
        maxVal = val[1]
        contrast_output = np.copy(output_int8)
        contrast_output[output_int8 < minVal] = 0
        contrast_output[output_int8 > maxVal] = 255
        image_object.set_data(contrast_output)
        fig.canvas.draw()

    def acceptClick(event):
        global accept, reject, end
        accept=True
        reject = False
        end=False
        plt.close()

    def rejectClick(event):
        global accept, reject, end
        accept=False
        reject = True
        end=False
        plt.close()

    def endClick(event):
        global accept, reject, end
        accept=False
        reject = False
        end=True
        plt.close()

    '''
    def update(val):
        global low_hough, up_hough, x, y, circle2, square2, square_rx, square_x, rx, ry, square_ry, square_y, centre_square
        low_hough = val[0]
        up_hough = val[1]
        wide = cv2.Canny(img, low_hough, up_hough)
        circles = cv2.HoughCircles(wide, cv2.HOUGH_GRADIENT, 3, int(size[0]/(2*binFactor)))
        if circles is not None:
            circles = np.round(circles[0, :]).astype("int")
            (x,y,r) = circles[0]
        rx = r-(int(r/10))
        ry = r-(int(r/10))
        circle2.set_center((x, y))
        circle2.width = 2*rx
        circle2.height = 2*ry
        square_rx = (rx**2 / 2)**0.5
        square_ry = (ry**2 / 2)**0.5
        square_x = x - square_rx
        square_y = y - square_ry
        square2.set_x(square_x)
        square2.set_y(square_y)
        square2.set_width(square_rx*2)
        square2.set_height(square_ry*2)
        centre_square.set_x(x - 10)
        centre_square.set_y(y - 10)
        fig.canvas.draw()
    '''
    
    text_box_x.on_submit(submitx)
    text_box_y.on_submit(submity)
    text_box_rx.on_submit(submitrx)
    text_box_ry.on_submit(submitry)
    bAccept.on_clicked(acceptClick)
    bReject.on_clicked(rejectClick)
    bEnd.on_clicked(endClick)
    #left_slider.on_changed(update)
    contrast_slider.on_changed(update_contrast)
    cid = fig.canvas.mpl_connect('button_press_event', onclickSquare)
    plt.show()

    #get the fraction
    fraction = ((square_ry * 2)*(square_rx * 2))/(size[0]*size[1])
    #add accept and reject + end buttons both here and on the angle one

    #fraction outside include in ImageNPS_ra

    left_val = int(binFactor*square_x)
    if left_val < 0:
        left_val = 0
    right_val = int(binFactor*(square_x + 2*square_rx))
    if right_val >= size[1]:
        right_val = size[1]-1
    top_val = int(binFactor*square_y)
    if top_val < 0:
        top_val = 0
    bottom_val = int(binFactor*(square_y + 2*square_ry))
    if bottom_val >= size[0]:
        bottom_val = size[0]-1

    return [left_val, right_val, top_val, bottom_val, filename, fraction, accept, reject, end]

'''
@njit
def setZero(img, x, y, r):
    size = img.shape
    new_image = np.zeros(size)
    count_all = 0.0
    count_left = 0.0
    for i in range(size[1]):
        for j in range(size[0]):
            count_all += 1
            dx = i - x
            dy = j - y
            if (dx**2 + dy**2) > r**2:
                new_image[j, i] = 0
            else:
                count_left += 1
    fraction = count_left/count_all
    return (new_image, fraction)
'''            

def theRealProcessing(path):
    num_avg = 10
    num_accept = 0
    overall = []
    file_list = [f for f in os.listdir(path) if (f.endswith(".mrc") and f.endswith("_PS.mrc") == False)]
    cryosparc = False
    for files in file_list:
        if "patch_aligned" in files or "rigid_aligned" in files:
            cryosparc = True
            break
    file_list2 = []
    if cryosparc == True:
        file_list2 = [f for f in file_list if ("patch_aligned" in f or "rigid_aligned" in f) and (f.endswith("aligned_doseweighted.mrc")==False)] 
        file_list = file_list2
    for i, filename in enumerate(file_list):
        mrc_blank = mrcfile.open(path + filename, permissive=True)
        data = mrc_blank.data
        mrc_blank.close()
        box_coords = cropImage2(data, filename)
        end = box_coords[8]
        #print("End: " + end)
        if end == True:
            num_accept = num_avg + 1
            break
        reject = box_coords[7]
        accept = box_coords[6]
        if accept == True:
            num_accept += 1
            overall.append(box_coords[0:6])
        if num_accept >= num_avg:
           break
    return overall
    

if __name__ == "__main__":
    path = sys.argv[1]
    write_dir = sys.argv[2]
    boxes = theRealProcessing(path)
    #now write angles to file
    with open(f"{write_dir}user_boxes.csv", 'w') as f:
        writer = csv.writer(f)
        writer.writerows(boxes)



import sys
import mrcfile
import numpy as np
import math
import csv
import os
from matplotlib.widgets import Slider
import matplotlib.pyplot as plt
import matplotlib as mpl
from PIL import Image, ImageEnhance

left_val = 0
right_val = 0
top_val = 0
bottom_val = 0

def cropImage(data, filename):
    global left_val, right_val, top_val, bottom_val
    size = data.shape
    #print(size)
    fig = plt.figure()
    ax = fig.subplots()
    left_val = 0
    right_val = size[1]
    top_val = 0
    bottom_val = size[0]

    left_x = [0, 0]
    left_y = [0, size[0]]
    right_x = [size[1], size[1]]
    right_y = [0, size[0]]
    top_x = [0, size[1]]
    top_y = [0, 0]
    bottom_x = [0, size[1]]
    bottom_y = [size[0], size[0]]
    p_left, = ax.plot(left_x, left_y, color="red", linewidth=1, alpha=0.5)
    p_right, = ax.plot(right_x, right_y, color="red", linewidth=1, alpha=0.5)
    p_top, = ax.plot(top_x, top_y, color="red", linewidth=1, alpha=0.5)
    p_bottom, = ax.plot(bottom_x, bottom_y, color="red", linewidth=1, alpha=0.5)

    normal_array = data/np.amax(data)
    img = Image.fromarray(np.uint8(normal_array*255), 'L')
    enhancer = ImageEnhance.Contrast(img)
    enhanced = enhancer.enhance(2)
    plt.imshow(enhanced, cmap = 'gray')

    plt.subplots_adjust(bottom=0.15, left=0.1, right=0.9, top=0.9)
    ax_bottom = plt.axes([0.15, 0.05, 0.65, 0.03])
    bottom_slider = Slider(ax_bottom, 'Bottom', valmin=0, valmax=size[0]-1, valinit=size[0]-1, valstep=2)
    ax_top = plt.axes([0.15, 0.95, 0.65, 0.03])
    top_slider = Slider(ax_top, 'Top', valmin=0, valmax=size[0]-1, valinit=0, valstep=2)
    ax_left = plt.axes([0.05, 0.25, 0.0225, 0.63])
    left_slider = Slider(ax_left, 'Left', valmin=0, valmax=size[1]-1, valinit=0, valstep=2, orientation="vertical")
    ax_right = plt.axes([0.9, 0.25, 0.0225, 0.63])
    right_slider = Slider(ax_right, 'Right', valmin=0, valmax=size[1]-1, valinit=size[1]-1, valstep=2, orientation="vertical")

    def on_press(event):
        sys.stdout.flush()
        global coords
        coords = event.key
        fig.canvas.mpl_disconnect(cid)
        plt.close(fig)
        return coords
    def update(val):
        global left_val, right_val, top_val, bottom_val
        left_val = int(left_slider.val)
        right_val = int(right_slider.val)
        top_val = int(top_slider.val)
        bottom_val = int(bottom_slider.val)
        #print(anglemin)
        #print(anglemax)
        if right_val < left_val:
            right_slider.set_val(left_slider.val+1)
            right_val = left_val+1
        if bottom_val < top_val:
            bottom_slider.set_val(top_slider.val+1)
            bottom_val = top_val+1
        left_x = [left_val, left_val]
        left_y = [0, size[0]]
        right_x = [right_val, right_val]
        right_y = [0, size[0]]
        top_x = [0, size[1]]
        top_y = [top_val, top_val]
        bottom_x = [0, size[1]]
        bottom_y = [bottom_val, bottom_val]
        #plt.clf()
        p_left.set_data(left_x,left_y)
        p_right.set_data(right_x,right_y)
        p_top.set_data(top_x,top_y)
        p_bottom.set_data(bottom_x,bottom_y)
        fig.canvas.draw()
    bottom_slider.on_changed(update)
    top_slider.on_changed(update)
    left_slider.on_changed(update)
    right_slider.on_changed(update)
    cid = fig.canvas.mpl_connect('key_press_event', on_press)
    plt.show()
    return [int(left_val), int(right_val), int(top_val), int(bottom_val), filename]

def theRealProcessing(path):
    num_avg = 1
    overall = []
    file_list = [f for f in os.listdir(path) if (f.endswith(".mrc") and f.endswith("_PS.mrc") == False)]
    cryosparc = False
    for files in file_list:
        if "patch_aligned" in files or "rigid_aligned" in files:
            cryosparc = True
            break
    file_list2 = []
    if cryosparc == True:
        file_list2 = [f for f in file_list if ("patch_aligned" in f or "rigid_aligned" in f) and (f.endswith("aligned_doseweighted.mrc")==False)] 
        file_list = file_list2
    for i, filename in enumerate(file_list):
        mrc_blank = mrcfile.open(path + filename)
        data = mrc_blank.data
        mrc_blank.close()
        box_coords = cropImage(data, filename)
        overall.append(box_coords)
        if i >= num_avg-1:
           break
    return overall
    

if __name__ == "__main__":
    path = sys.argv[1]
    write_dir = sys.argv[2]
    boxes = theRealProcessing(path)
    #now write angles to file
    with open(f"{write_dir}user_boxes.csv", 'w') as f:
        writer = csv.writer(f)
        writer.writerows(boxes)



import setuptools
from setuptools.command.egg_info import egg_info

class egg_info_ex(egg_info):
    """Includes license file into `.egg-info` folder."""

    def run(self):
        # don't duplicate license into `.egg-info` when building a distribution
        if not self.distribution.have_run.get('install', True):
            # `install` command is in progress, copy license
            self.mkpath(self.egg_info)
            self.copy_file('LICENSE.txt', self.egg_info)

        egg_info.run(self)


with open("README.md", 'r', encoding="utf-8") as fh:
    long_description = fh.read()


setuptools.setup(
    name="magCalEM",
    version="1.3.3",
    author="Joshua L. Dickerson",
    author_email="jdickerson@mrc-lmb.cam.ac.uk",
    description="A program to calibrate the pixel size of cryoEM data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    license="MIT",
    license_files = ('LICENSE.txt',),
    install_requires=['PyQt5', 'numpy', 'mrcfile', 'opencv-python', 'matplotlib', 'scipy', 'numba', 'psutil', 'scikit-image', 'pandas', 'lmfit'],
    cmdclass = {'egg_info': egg_info_ex},
)



import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
import os
import sys
import mrcfile
import scipy
import csv
import math
from matplotlib.widgets import Slider
from scipy import ndimage
from scipy.signal import savgol_filter
from scipy.optimize import curve_fit
from scipy.optimize import minimize
from scipy.optimize import minimize_scalar
from PIL import Image, ImageEnhance
from multiprocessing import Pool
from numba import jit
from functools import partial

savgol_window = 27

def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx

def ellipseEqu(angles, a, b, angle_a):
    return (a*b)/((b*np.cos(angles-angle_a))**2+(a*np.sin(angles-angle_a))**2)**0.5

def radial_profile_start(data, center):
    y,x = np.indices((data.shape)) # first determine radii of all pixels
    #yx = complex(y, x) #make complex
    all_angles = np.angle(x-center[0] + 1j*(y-center[1]), deg=True)+180 #get angles
    #excluded_indices = np.argwhere(all_angles < angles[0]) #find indices outside this angular range
    #to test

    r = np.sqrt((x-center[0])**2+(y-center[1])**2)
    ind = np.argsort(r.flat) # get sorted indices
    sr = r.flat[ind] # sorted radii
    sim = data.flat[ind] # image values sorted by radii
    sangles = all_angles.flat[ind] #sort angles in same way
    return [sr, sangles, sim]

def radial_profile(sr, sangles, sim, angles):

    excluded_indices = np.argwhere(sangles < angles[0])
    excluded_indices2 = np.argwhere(sangles > angles[1])
    
    sim2 = sim.copy()
    sim2[excluded_indices] = 0 # image values sorted by radii
    sim2[excluded_indices2] = 0 # image values sorted by radii
    ri = sr.astype(np.int32) # integer part of radii (bin size = 1)
    # determining distance between changes
    deltar = ri[1:] - ri[:-1] # assume all radii represented
    rind = np.where(deltar)[0] # location of changed radius
    nr = rind[1:] - rind[:-1] # number in radius bin
    csim = np.cumsum(sim2, dtype=np.float64) # cumulative sum to figure out sums for each radii bin
    tbin = csim[rind[1:]] - csim[rind[:-1]] # sum for image values in radius bins
    radialprofile = tbin/nr # the answer

    the_r = sr[rind]
    
    return [abs(radialprofile), the_r]

def method2(angle, img, this_step, lattice_res, sr, sangles, sim, pxGuess, aliased):
    size = img.shape
    leeway_plus = 0.04
    leeway_minus = 0.04
    angles = (angle - (this_step/2), angle + (this_step/2))
    #print(size[0])
    size_alias =   2**(int(math.log(size[0], 2)))  
    minradi = int((size[0]*(pxGuess-leeway_minus))/lattice_res)
    maxradi = int((size[0]*(pxGuess+leeway_plus))/lattice_res)
    if aliased:
        minradi = int((size_alias*(pxGuess-leeway_minus))/lattice_res)
        maxradi = int((size_alias*(pxGuess+leeway_plus))/lattice_res) 
    radi, radi2 = minradi, maxradi

    result = radial_profile(sr, sangles, sim, angles)
    rad = result[0]
    smoothed = savgol_filter(rad, savgol_window, 2)
    ri = result[1]

    max_index = np.argmax(smoothed[radi:radi2])
    the_max = ri[max_index + radi]
    min_index = np.argmin(smoothed[radi:radi2])
    the_min = ri[min_index + radi]

    px = (the_max/(size[0]))*lattice_res
    if aliased:
        px = (the_max/size_alias)*lattice_res
    
    if (max_index <= 1):
        px = 0
    return px

def getAnisoFit(angles, px_all, write_dir, print_text):
    #get nonzeros
    #np.count_nonzero(px_all, axis=0)
    #now convert zeroes to nans
    px_all2 = np.array(px_all)
    nan_array = np.where(px_all2>0.0, px_all2, np.nan)
    mean_px = np.nanmean(nan_array, axis=0)
    nan_mean_px = np.where(mean_px>0.0, mean_px, np.nan)
    mean_px_all = np.nanmean(nan_mean_px)
    print("Average px size is " + str(mean_px_all))
    print_text.append("Average px size is " + str(mean_px_all))
    stdev = np.nanstd(nan_mean_px)
    #d_from_mean = abs(mean_px-mean_px_all)
    max_deviations = 2
    #not_outlier = d_from_mean < max_deviations * stdev
    #px_sizes = mean_px[not_outlier]
    #mean_px = np.mean(px_all, axis=0)
    px_sum = []
    #print("got here")
    angles_rad = []
    for i, angle in enumerate(angles):
        if np.isnan(mean_px[i]) == False and mean_px[i] > 0.0:
            #should exclude outliers as well
            #print("gets here")
            d_from_mean = abs(mean_px[i]-mean_px_all)
            if d_from_mean < max_deviations*stdev:
                #print("anf here")
                angles_rad.append(np.radians(angle))
                px_sum.append(mean_px[i])
    #print("managed this")
    for i in range(len(angles_rad)):
        angles_rad.append(angles_rad[i] + np.pi)
        px_sum.append(px_sum[i])
        
    #print(px_sum)
    
    #print("got mean px")
    #print(str(px_sum))
    this_savgol = 37
    smoothed = savgol_filter(px_sum, this_savgol, 2, mode='wrap')
    max_index = np.argmax(smoothed)
    a = smoothed[max_index]
    angle_a = angles_rad[max_index]
    if angle_a >= np.pi:
        angle_a -= np.pi
    angle_b = angle_a - np.pi/2
    if angle_b < 0:
        angle_b = np.pi + angle_b

    min_index = find_nearest(angles_rad, angle_b)
    b = smoothed[min_index]
    
    
    popt2, pcov2 = curve_fit(ellipseEqu, angles_rad, px_sum, p0=[a, b, angle_a])
    refined_r_raw = ellipseEqu(angles_rad, *popt2)

    with open(f"{write_dir}aniso_params.csv", 'w') as f:
        f.write(f"{str(popt2[0])},{str(popt2[1])},{str(popt2[2])}")
    print("Finished fitting elliptical parameters")
    print_text.append("Finished fitting elliptical parameters")

    threshold = 0.002
    print(f"Pixel size on major axis is {str(popt2[0])}A")
    print_text.append(f"Pixel size on major axis is {str(popt2[0])}A")
    print(f"Pixel size on minor axis is {str(popt2[1])}A")
    print_text.append(f"Pixel size on minor axis is {str(popt2[1])}A")
    dif = abs(popt2[0]-popt2[1])
    avg_r = (popt2[0]+popt2[1])/2
    percent = 100*(dif / avg_r)
    if percent >= 1.0:
        print("Correcting anisotropy is recommended")
        print_text.append("Correcting anisotropy is recommended")
    else:
        print("Anisotropy is low enough")
        print_text.append("Anisotropy is low enough")

    #finally write the printText file
    with open(f"{write_dir}outputText.txt", 'a') as f:
        for line in print_text:
            f.write(f"{line}\n")
    

def runScript(path, temperature, pxGuess, cores, aliased, spectra_dir, ps_dir, write_dir, latticeType, latticeRes):
    print_text = []
    print("Measuring pixel sizes across all angles")
    print_text.append("Measuring pixel sizes across all angles")
    gold_lattice_param = temperature*5.67075E-5 + 4.06111 #A 
    if temperature < 43.0:
        gold_lattice_param = 43.0*5.67075E-5 + 4.06111 #A       
    goldLattice_111 = gold_lattice_param / (1**2+1**2+1**2)**0.5
    goldLattice_200 = gold_lattice_param / (2**2)**0.5
    #gc_res = 3.3378
    gc_res = 3.4187
    lattice_res = goldLattice_111
    if latticeType == "gold-111":
        lattice_res = goldLattice_111
    elif latticeType == "gold-200":
        lattice_res = goldLattice_200
    elif latticeType == "graphitized-carbon":
        lattice_res = gc_res
    elif latticeType == "define_lattice_constant":
        lattice_res = latticeRes

    step = int(2)
    angles = np.arange(1, 180, step)

    #get directory
    '''
    NWPS_dir = f"{path}{spectra_dir}/avg/10/"
    file_list = []
    if aliased == False:
        file_list = [f for f in os.listdir(NWPS_dir) if (f.endswith(".mrc") and f.endswith("aliased.mrc") == False)]
    else:
        file_list = [f for f in os.listdir(NWPS_dir) if f.endswith("aliased.mrc")]
    num_files = len(file_list)

    if num_files < 1:
        NWPS_dir = f"{path}{spectra_dir}/avg/"
        file_list = [f for f in os.listdir(NWPS_dir) if f.endswith(".mrc")]
        if aliased == False:
            file_list = [f for f in os.listdir(NWPS_dir) if (f.endswith(".mrc") and f.endswith("aliased.mrc") == False)]
        else:
            file_list = [f for f in os.listdir(NWPS_dir) if f.endswith("aliased.mrc")]
        num_files = len(file_list)
    '''
    NWPS_dir = ps_dir
    file_list = [f for f in os.listdir(NWPS_dir) if f.endswith(".mrc")]
    num_files = len(file_list)
    new_write_dir = f"{write_dir}pxSizes/"
    #os.mkdir(new_write_dir)
    #new_write_dir += '/'

    print(f"{num_files} file(s) to process")
    print_text.append(f"{num_files} file(s) to process")
    px_all = []
    for i, filename in enumerate(file_list):
        this_image_px = []
        mrc = mrcfile.open(NWPS_dir+filename)
        img = mrc.data
        mrc.close()
        size = img.shape
        center = (size[0]/2, size[1]/2)
        sr, sangles, sim = radial_profile_start(img, center)

        p = Pool(processes=cores)
        method2_1 = partial(method2, img=img, this_step=step, lattice_res=lattice_res, sr=sr, sangles=sangles, sim=sim, pxGuess=pxGuess, aliased=aliased)
        px_sizes = p.map(method2_1, angles)
        p.close()

        overall = zip(angles, px_sizes)
        #with open(f"{NWPS_dir}{filename[:-4]}_pxSizes.csv", 'w') as f:
        with open(f"{new_write_dir}{filename[:-4]}_pxSizes.csv", 'w') as f:
            writer = csv.writer(f)
            writer.writerows(overall)
        
        px_all.append(px_sizes)
        print(f"Image {str(i+1)}/{str(num_files)} complete")
        print_text.append(f"Image {str(i+1)}/{str(num_files)} complete")
    print("Angular dependent pixel size measurements complete")
    print_text.append("Angular dependent pixel size measurements complete")

    #Next step is to fit the ellipse to get the aniso params
    print("Getting elliptical parameters")
    print_text.append("Getting elliptical parameters")
    getAnisoFit(angles, px_all, write_dir, print_text)

     
     
    
    

